/*
History
=======
Ver. 1.0  2003-10-26  Initial release.
Ver. 1.1  2003-10-26  Cosmetic changes; selected date is now colored.
Ver. 1.2  2003-10-26  A minor but embarrassing bug displayed the wrong month name.
Ver. 1.3  2003-10-26  Fixed bug relating to the week number for the few days of the year
                      following a year having 53 week numbers.
Ver. 1.4  2003-10-27  Added tool tips to form, change of background color for selected date;
                      made midnight hour be 12, not 0;
                      added toggling of display for both date fields (YYYY-MM-DD < - > DD-MM-YYYY
                      and DD-MonName-YYYY < - > DD-MonthName-YYYY);
                      added toggling of time display between am/pm and 24hr modes.
Ver. 1.5  2003-10-28  Fixed display of 24hr time to have leading zero for hours midnight through 0900;
                      added code to update calendar to current date when midnight is detected;
                      NOTE: calendar can't be changed for 15 secs. to allow detection of clock change
                      (sometimes CPU usage freezes the browser).
Ver. 1.6  2003-10-30  Improved cosmetics and Javascript/HTML compatibility (thanks to JoeFlash);
                      Added buttons to increment/decrement date by decades;
                      Limited first year of calendar to 1783, the first full year of the Gregorian Calendar.
Ver. 1.7  2003-11-01  Fixed display bug when 31st day was selected and "next month" button clicked;
                      the very next month was being skipped (this is actually a Javascript bug)
Ver. 1.8  2003-11-08  Added local sunrise, solar noon, and sunset times;
                      Added Date Calculation section.
Ver. 1.9  2003-11-09  Simplified code; reduced size of sidebar a little;
                      Added link to time zone map;
                      Allowed for +14 UTC offset for the island of Kiribati -
                      see http://wwp.greenwichmeantime.com/time-zone/ for
                      time zone information.
Ver. 1.10 2003-11-10  Added "Load Settings" button
Ver. 1.11 2003-11-11  Added current time for the specified time zone;
                      fixed minor bug re labels of buttons on startup;
                      fixed "Load Settings" to refresh sun time displays;
                      changed link to MultiMap for finding geo coordinates;
                      changed time zone sign to accord with most time zone maps;
                      removed gradient background because it caused display problems when scrolling.
Ver. 1.12 2003-11-14  Modified to allow multilanguage adaptation;
                      all English text that shows in the user inteface can be
                      set in the file en.ini
Ver. 1.13 2003-11-20  Revised to make everything work even if settings.ini and
                      language INI files are missing;
                      tidied up code, simplifying some functions and CSS;
                      formatting of Sun times is now in the main code;
                      made sidebar slimmer, shorter; made CSS more ordered;
                      added forward/back by 100 years by double-clicking on end-buttons.
Ver. 1.14 2003-12-02  Fixed minor bug in timezone adjustment for value 0.25 (I'm not
                      aware of any timezones with only a .25 hr adjustment)
Ver. 1.15 2003-12-07  Changed calendar increment/decrement function to fire on
                      Ctrl-Click rather than double click;
                      fixed minor bug in fBuildSelect() function (didn't affect
                      any calculations); Added Help page.
Ver. 1.16 2004-05-12  Fixed typo in tooltip; Changed Countdown to Date function to
                      show negative days if the selected date is in the past;
                      Fixed (I believe) a bug in the Locale time display of the
                      date that occurred only at month end;
                      Added 24 hr time display mode for sun times;
                      Added "Use local time offset" button to location setting.
Ver. 1.17 2004-07-08  Made MaxThon compatible; added French language file, FR.INI
Ver. 1.18 2004-12-11  Added option in plugin.ini to start the calendar on Monday
Ver. 1.19 2004-12-12  Fixed version number in plugin.ini; changed cursor to finger for
                      Monday/Sunday toggle when hovering over the day names.
Ver. 1.20 2005-10-16  BETA: Major new functionality added; Sun Twilight times and position
                      information, Moon rise/set times and position & phase information,
                      time for over 4000 pre-defined cities, a user-defined number of
                      date calculator and world time rows, and holiday and user-defined
                      calendar markers.
Ver. 1.21 2005-10-27  Bug fixes and minor changes to v. 1.20 (Beta); moved history to README.TXT
Ver. 1.21a 2005-10-29 Bug fixes and minor changes to v. 1.21; many thanks to Cosmolit for pointing them out.
Ver. 1.21b 2005-10-29 Bug fixes and minor changes to v. 1.21a;
                      many thanks to Cosmolit and npo33770 for pointing out more bugs.
Ver. 1.21c 2005-10-30 Bug fixes and minor changes.
Ver. 1.21d 2005-10-30 Implemented workaround for 64KB limit for INI files in some versions of Windows.
Ver. 1.21e 2005-10-30 Fixed bug in workaround for 64KB limit for INI files in some versions of Windows.
Ver. 1.22 2005-10-31  Added user defined selection of how Help is displayed (popup or tabbed window);
                      changed font weight of selected and today's dates to be bold;
                      if today is a holiday or personal reminder date,
                      the background color is green or pink, not blue.
Ver. 1.22a 2005-10-31 Fixed time zone information for a number of locations with 0 offset
Ver. 1.22b 2005-12-24 Cosmetic changes in CSS to control width of display; width:190px
*/

//==================================
// global values & general functions
//==================================

var vglobalDate;           // Global for the current selected date; startup value is today's date
var kCalendarStart = 1583; // 1st full year of Gregorian calenadar; not adopted in England until 1752

//==============================
// Main initialization functions
//==============================

function fInitAllForms() {
  // these functions use global forms not passed as arguments to this function
  vglobalDate = new Date();      // set global date object with today's date
  fFillInCalendar(frmCalendar);  // fillin calendar based on first day of month
  fUpdateAllCalculations();
  fReStyleCalendar(frmCalendar);
//  fSetTextbox(frmCalendarData.frmCalDataDiaryInput, 'italic', 'center', msgDiaryEvent);
}

function fUpdateAllCalculations() {
  // these functions use global forms not passed as arguments to this function
  fDisplayDerived(frmCalendarData);  // display variety of info derived from date
  fSunInfoUpdate(frmLocation, frmSunTimes);
  fMoonInfoUpdate(frmLocation, frmMoonTimes);
  fUpdateDateCalculations(frmDateCalc);
}

function fFillInCalendar(argCalForm) {
  with (document) {
    // find the end of the month
    var vTempDate = vglobalDate;
    with (new Date(vTempDate)) {
      setDate(32);
      var vEoM = 32 - getDate();
    }
    // write day names
    for (var vIndex = 0; vIndex < 7; vIndex++) {
      argCalForm['LB'+vIndex].value = kShortDayName[vIndex];
    }

    // fillin all valid dates
    var vDayNum;
    for (var vIndex = 0; vIndex < 42; vIndex++) {
      vDayNum = vIndex - fCalendarPad(vglobalDate) + 1;
      argCalForm['BB'+vIndex].value = (vDayNum < 1 || vDayNum > vEoM) ? '  ' : vDayNum;
    }

    // set month and year for calendar heading
    argCalForm.frmCalendarPickedMonth.value = kLongMonthName[vglobalDate.getMonth()]
                                              + ' '+ vglobalDate.getFullYear() + '';

    fClearDayStyle(argCalForm); // clear day colors and holiday tooltips
    fColorToday(argCalForm);
    fColorHolidays(argCalForm);
    fSetEasterDates(frmCalendar, frmCalendarData);
    fCheckDiary(argCalForm, frmCalendarData);

    if (vglobalMondayStart == 1) fRotateCalendar(argCalForm);
  }
}

function fRotateCalendar(argCalForm) {
  // rotate labels one slot if Monday start
  with (document) {
    var vTemp = argCalForm['LB'+6].value;
    for (var vIndex = 6; vIndex > 0; vIndex--) {
      argCalForm['LB'+vIndex].value = argCalForm['LB'+(vIndex-1)].value;
    }
    argCalForm['LB'+0].value = vTemp;
  }
}

//======================
// Date Picker Functions
//======================

function fDayClick(argDate, argCalForm) {
  // when a date is clicked, update the global date & redisplay everything
  if (+argDate == 0) return;     // ignore clicks not on a displayed date
  
  vglobalDate.setDate(+argDate); // set global date to picked date

  fClearDayStyle(argCalForm);    // clear day colors and holiday tooltips

  fColorToday(argCalForm);
  fColorHolidays(argCalForm);
  fSetEasterDates(frmCalendar, frmCalendarData);
  fColorSelectedDay(argCalForm); // MUST come AFTER fColorToday, fColorHolidays, fSetEasterDates

  fCheckDiary(argCalForm, frmCalendarData);

  fUpdateAllCalculations();
}

function fNow(argClick) {
  // on any change to the calendar, update month & year for calendar heading
  vglobalDate = new Date('1 ' + argClick.value);
}

function fIncDecMonth(argClick) {
  // increment or decrement month/year when arrow button clicked
  if (window.event.ctrlKey) {
    switch (Math.abs(argClick)) {
      case 120 : argClick *= 10; break; // change to 100 years if CTRL-click on 10 year arrows
      case  12 : argClick *=  5; break; // change to   5 years if CTRL-click on  1 year arrows
      case   1 : argClick *=  6; break; // change to  6 months if CTRL-click on 1 month arrows
    }
  }

  vglobalDate.setDate(1); // fixes problem of incrementing 31-day months when last day was selected
  vglobalDate.setMonth(vglobalDate.getMonth()+argClick);
  
  if (vglobalDate.getFullYear() < kCalendarStart) { // limit earliest year to kCalendarStart
    vglobalDate = new Date(kCalendarStart, 0, 1);
    alert (kCalendarStart + ' ' + msgOutOfCalendarRange)
  }
}

function fSetDayStyle(argIndex, argColor, argBGColor, argFontWeight, argCalForm) {
  // set text color & background color of one date to specified value
  with (document) {
    if (argColor == kHolidayColor && argCalForm['BB'+argIndex].style.fontWeight == 'bold') {
      // do nothing if the text is bold because it was today's date
    } else {
      argCalForm['BB'+argIndex].style.fontWeight = argFontWeight;
    }
    argCalForm['BB'+argIndex].style.color           = argColor;
    argCalForm['BB'+argIndex].style.backgroundColor = argBGColor;
  }
}

function fClearDayStyle(argCalForm) {
  // reset text color & background color to default values; weekends have different background
  vSpecialCol1 = (vglobalMondayStart == 1 ? 0 : 5);
  vSpecialCol2 = 6;

  for (var vIndex = 0; vIndex < 42; vIndex++) {
    argCalForm['BB'+vIndex].title = '';  // clear Holiday tooltips
    
    if (vIndex%7 == vSpecialCol1 || vIndex%7 == vSpecialCol2) { // color Saturday/Sunday columns differently
      fSetDayStyle(vIndex, kNormalDayColor, kWeekEndColor, 'normal', argCalForm);
    } else {
      fSetDayStyle(vIndex, kNormalDayColor, kNormalDayBgColor, 'normal', argCalForm);
    }
  }
}

function fColorToday(argCalForm) {
  // set different color & background for today's date
  var vTempDate = new Date();
  with (vTempDate) {
    if ((getMonth() == vglobalDate.getMonth()) && (getYear() == vglobalDate.getYear())) {
      fSetDayStyle((fCalendarPad(vTempDate) + getDate() - 1), kTodayColor, kTodayBgColor, 'bold', argCalForm);
    }
  }
}

function fColorSelectedDay(argCalForm) {
  // set different color & background for selected day
  fSetDayStyle((fCalendarPad(vglobalDate) + vglobalDate.getDate() - 1), kSelectedDayColor, kSelectedDayBgColor, 'bold', argCalForm);
}

function fColorDate(argCalForm, argDate, argColor, argBGColor) {
  // set different color & background for selected date
  fSetDayStyle((fCalendarPad(vglobalDate) + argDate - 1), argColor, argBGColor, 'normal', argCalForm);
}

function fCalendarPad(argDate) {
  // calculate the number of 'blank days' at the calendar month start
  var vPad = (argDate.getDay() - argDate.getDate() + 77 )%7;
  if (vglobalMondayStart == 1) { vPad++; } // add 1 for a Monday start
  return (vPad);
}

function fReStyleCalendar(argCalForm) {
  // when calendar is collapsed, change heading to show current date in current style
  with (document) {
    with (argCalForm) {
      var vNow = new Date();
      with (vNow) {
        if (frmCalendarBody.style.display == 'block') {
           frmCalendarPickedMonth.value  = kLongMonthName[vglobalDate.getMonth()] + ' '+ vglobalDate.getFullYear() + '';
        } else {
          switch (vglobalMonthStyle) {
            case 0: frmCalendarPickedMonth.value = getDate() + ' ' + kLongMonthName[getMonth()]  + ' '+ getFullYear() + ''; break;
            case 1: frmCalendarPickedMonth.value = getDate() + ' ' + kShortMonthName[getMonth()] + ' '+ getFullYear() + ''; break;
            case 2: frmCalendarPickedMonth.value = kLongMonthName[getMonth()]  + ' ' + getDate() + ', ' + + getFullYear() + ''; break;
            case 3: frmCalendarPickedMonth.value = kShortMonthName[getMonth()] + ' ' + getDate() + ', ' + + getFullYear() + ''; break;
          }
        }
      }
    }
  }
}

function fBuildDatePicker(argCalForm) {
  // create buttons for day names
  document.write("\<tr>\<td>");
  for (var vIndex = 0; vIndex <= 6; vIndex++) {
    document.write("\<input type=\"text\" name=\"LB"+vIndex +"\" id=\"frmLabelDayName"+ vIndex+ "\" "+
    " size=\"2\" readonly=\"readonly\" title=\"Click to toggle between Sunday and Monday start\" "+
    " onClick=\"vglobalMondayStart=fCycleStyle(vglobalMondayStart, 'MondayStart', 2)\;"+
    " fFillInCalendar(frmCalendar)\;\" class=\"CalDayLabels\" value=\"??\">");
    if (vIndex%7<6) document.write("\<\/td>\<td>");
  }
  document.write("\<\/td>\<\/tr>\<tr>");

  // create buttons for days
  for (var vIndex=0; vIndex<42; vIndex++) {
    document.write("\<td>\<input type=\"text\" name=\"BB" + vIndex + "\" "+
    " size=\"2\" readonly=\"readonly\" class=\"CalDays\""+
    " title=\"\" value=\"??\" onClick=\"fDayClick(this.value," + argCalForm.id + ")\; \">\<\/td>");
    if (vIndex%7==6) document.write("\<\/tr>\<tr>");
  }
  document.write("\<\/td>\<\/tr>");
}

//============================
// Calendar reminder functions
//============================

function fCheckDiary(argCalForm, argCalData) {
  with (argCalData) {
    var vPad   = fCalendarPad(vglobalDate) - 1;
    var vEntry = argCalForm['BB' + (vPad + vglobalDate.getDate())].title;
    if (vEntry != '') {
      fSetTextbox(frmCalDataDiaryInput, 'normal', 'center', vEntry);
      frmCalDataDiarySet.checked = true;
    } else {
      fSetTextbox(frmCalDataDiaryInput, 'italic', 'center', msgDiaryEvent);
      frmCalDataDiarySet.checked = false;
    }
  }
}

function fDiaryEdit(argForm) {
  with (argForm) {
    fSetTextbox(frmCalDataDiaryInput, 'normal', 'left', '');
    if (frmCalDataDiaryInput.value == msgDiaryEvent) {
      frmCalDataDiaryInput.value = '';
    }
  }
}

function fSaveDate(argForm) {
// Save selected date & message as Month (1-12, with January=1),Day (1-31),Message Text
  with (argForm) {
    if (frmCalDataDiarySet.checked) {
      if (frmCalDataDiaryInput.value == msgDiaryEvent) { // ignore clicks when default msg is displayed
        frmCalDataDiarySet.checked = false;
      } else {
        fWriteINI(kSpecialDatesFile, String(vglobalDate.getMonth() + 1), String(vglobalDate.getDate()), frmCalDataDiaryInput.value);
        fDayClick(vglobalDate.getDate(), frmCalendar); // update calendar
      }
    } else {
      fWriteINI(kSpecialDatesFile, String(vglobalDate.getMonth() + 1), String(vglobalDate.getDate()), ''); // clear entry if unchecked
      frmCalDataDiaryInput.value = '';
    }
  }
}

//===========================
// Holiday calendar functions
//===========================

function fMarkDate(argCalForm, argPad, argDate, argMessage, argColor, argBGColor) {
  // make the holiday name a tooltip and color the background
  fColorDate(argCalForm, argDate, argColor, argBGColor);
  if (argCalForm['BB' + (argPad + argDate)].title == '') {
    argCalForm['BB' + (argPad + argDate)].title = argMessage;
  } else {
    argCalForm['BB' + (argPad + argDate)].title += ', '+ argMessage;
  }
}


function fColorHolidays(argCalForm) {
  // read INI file to find Relative Date and Fixed Date Holidays;
  // color corresponding calendar dates and set corresponding tooltips to Holiday name
  var vCurrentMonth = vglobalDate.getMonth() + 1;
  var vPad          = fCalendarPad(vglobalDate) - 1;
  
  var vHoliday;
  for (var vIndex = 1; vIndex; vIndex++) { // Loop until no more relative date holidays to read
    var vParamArray = fReadINI(kSpecialDatesFile, 'RelativeDateHolidays', String(vIndex), 'None').split(',');
    if (vParamArray[0] != 'None') {
      if (vCurrentMonth == +vParamArray[0]) { // color date on calendar if holiday is in the current month
        if (+vParamArray[2] == 0) vParamArray[0] = +vParamArray[0] + 1; // special rule for WeekDay=0
        vHoliday = fNthWeekDay(vglobalDate.getFullYear(), vParamArray[0], vParamArray[1], vParamArray[2]);
        fMarkDate(argCalForm, vPad, vHoliday[1], vParamArray[3], kHolidayColor, kHolidayBGColor);
      }
    } else {
      break;
    }
  }

  for (var vIndex = 1; vIndex; vIndex++) { // Loop until no more fixed date holidays to read
    var vParamArray = fReadINI(kSpecialDatesFile, 'FixedDateHolidays', String(vIndex), 'None').split(',');
    if (vParamArray[0] != 'None') {
      if (vCurrentMonth == +vParamArray[0]) { // color date on calendar if holiday is in the current month
        fMarkDate(argCalForm, vPad, +vParamArray[1], vParamArray[2], kHolidayColor, kHolidayBGColor);
      }
    } else {
      break;
    }
  }

  var vCurrentMonthLength = fDaysInMonth(vglobalDate.getYear(), vCurrentMonth);
  for (var vIndex = 1; vIndex <= vCurrentMonthLength; vIndex++) { // Loop until no more fixed user-defined dates to read for the month
    var vMessage = fReadINI(kSpecialDatesFile, String(vCurrentMonth), String(vIndex), '');
    if (vMessage != '') {
      fMarkDate(argCalForm, vPad, vIndex, vMessage, kHolidayColor, kUserDateBGColor);
    }
  }
}

function fUpdateEasterDates(argCalForm, argDateForm) {
  vglobalEasterStyle[0] = fCycleStyle(vglobalEasterStyle[0], 'EasterStyle', vglobalEasterStyle[1]);
  fSetEasterDates(argCalForm, argDateForm);
}

function fSetEasterDates(argCalForm, argDateForm) {
  var vCurrentMonth = vglobalDate.getMonth() + 1;
  var vPad          = fCalendarPad(vglobalDate) - 1;

  var vEasterArray  = fEasterDate(vglobalDate.getFullYear());
  var vJDay         = fJDayFromYMD(vglobalDate.getFullYear(), vEasterArray[0], vEasterArray[1]);

  if (vCurrentMonth == vEasterArray[0]) {  // color date on calendar if holiday is in the current month
    fMarkDate(argCalForm, vPad, vEasterArray[1], msgEaster, kHolidayColor, kHolidayBGColor);
  }
  vEasterArray      = [msgEaster + ': ' + vEasterArray[1] + ' ' + kLongMonthName[vEasterArray[0]-1]];

  var vDate;
  var vEntry;
  for (var vIndex=1; vIndex; vIndex++) { // Loop until no more Easter related dates to read
    var vParamArray = fReadINI(kSpecialDatesFile, 'EasterBasedHolidays', String(vIndex), 'None').split(',');
    if (vParamArray[0] != 'None') {
      vDate     = fYMDFromJDay(vJDay + (+vParamArray[0]));
      vEntry    = vParamArray[1]  + ': ' + vDate[2] + ' ' + kLongMonthName[vDate[1]-1];
      vEasterArray.push(vEntry);  // put in array

      if (vCurrentMonth == vDate[1]) { // color date on calendar if holiday is in the current month
        fMarkDate(argCalForm, vPad, vDate[2], vParamArray[1], kHolidayColor, kHolidayBGColor);
      }
    } else {
      break;
    }
  }

  var vAdventArray = fAdventDate(vglobalDate.getFullYear());
  if (vCurrentMonth == vAdventArray[0]) { // color date on calendar if holiday is in the current month
    fMarkDate(argCalForm, vPad, vAdventArray[1], msgAdvent, kHolidayColor, kHolidayBGColor);
  }

  // add Advent date; closest Sunday to Nov. 30; adjust Advent month for proper month name lookup
  vEasterArray.push(msgAdvent + ': ' + vAdventArray[1] + ' ' + kLongMonthName[vAdventArray[0]-1]);

  vglobalEasterStyle[1] = vEasterArray.length;
  argDateForm.frmCalDataEaster.value = vEasterArray[vglobalEasterStyle[0]];
}

//===============================
// Display Derived Date functions
//===============================

function fDisplayDerived(argForm) {
  with (document) {
    if (argForm.style.display == 'none') {return;} // skip if form isn't displayed

    // compute basic date units
    var vYear  = vglobalDate.getFullYear();
    var vMonth = vglobalDate.getMonth();
    var vDay   = vglobalDate.getDate();

    // calculate day of year and days to end of year
    var vDOYArray = fDOY(vYear, vMonth, vDay);  // gives day of year, remaining days

    // Use ISO8601 standard to calculate weeks in a year, the current week number,
    // and the week day number (with Monday = 1)
    var vWeekInfo      = fYWDFromYMD(vYear, vMonth, vDay); // gives Year, Week Number, Day of Week Number
    var vWeeksInYear   = fWeeksInYear(vWeekInfo[0]);

    with (argForm) { // Populate fields in form
      frmCalDataDateYMD.value = fDateDisplay([vYear, vMonth+1, vDay], vglobalDateStyle);

      switch (vglobalMonthStyle) {
        case 0: frmCalDataDateDMY.value = vDay +' '+ kLongMonthName[(vMonth)]  +' '+ vYear; break;
        case 1: frmCalDataDateDMY.value = vDay +' '+ kShortMonthName[(vMonth)] +' '+ vYear; break;
        case 2: frmCalDataDateDMY.value = kLongMonthName[(vMonth)]  + ' ' + vDay + ', ' + vYear; break;
        case 3: frmCalDataDateDMY.value = kShortMonthName[(vMonth)] + ' ' + vDay + ', ' + vYear; break;
      }

      frmCalDataDOY.value        = msg_Day + ' ' + vDOYArray[0] + ' ' + msg_of + ' '
                                 + (vDOYArray[0] + vDOYArray[1]) + ' (' + vDOYArray[1] + ' '+ msg_left + ')';

      frmCalDataWeekDay.value    = kLongDayName[vWeekInfo[2]-1] + ' (ISO '+ msg_Day + ' ' + vWeekInfo[2] + ') ';

      frmCalDataWeekNumber.value = msg_Week +' ' + vWeekInfo[1] + ' '+ msg_of +' ' + vWeeksInYear
                                   + ' (' + (vWeeksInYear - vWeekInfo[1])+' '+ msg_left +')';
    }
  }
}

//================
// Clock functions
//================

function fShowTime(argFrmClock) {
  // compute a time output, & output once per second (recursive function)
  with (document) {
    var vNow = new Date();
    with (vNow) {
      var vHours   = getHours();
      var vMinutes = getMinutes();
      var vSeconds = getSeconds();

      with (argFrmClock) {
        switch (+vglobalClockStyle) {
          case 0:   // fix hours display to be 1-12, & midnight display to be 12 hours, not 0 hours
            var vDisplayHours = (vHours > 12) ? vHours - 12 : (vHours == 0) ? 12 : vHours;
            frmClockOutput.value = fHMSTimeString(vDisplayHours, vMinutes, vSeconds, false)
                                   + ((vHours >= 12) ? ' ' + msg_pm : ' ' + msg_am);
            break;

          case 1:  // 24 hr mode
            frmClockOutput.value = fHMSTimeString(vHours, vMinutes, vSeconds, true);
            break;

          case 2:  // Internet beat time
            frmClockOutput.value = fInternetBeatTime(vNow);
            break;

          case 3:  // UTC time
            frmClockOutput.value = vNow.toUTCString().substr(16, 28);
            break;
        }
      }
      
      // if within 5 seconds after midnight, reset calendar to new date
      if ((vHours == 0) && (vMinutes == 0) && ((vSeconds >= 0) || (vSeconds <= 5))) {
        fInitAllForms();
      }
      
      // every minute: update Locale time & displayed Sun/Moon forms, & check alarms
      if (vSeconds > 0 && vSeconds <= 1) { 
        fLocaleTime(frmLocation, frmTimeZones)
        fSunInfoUpdate(frmLocation, frmSunTimes);
        fMoonInfoUpdate(frmLocation, frmMoonTimes);
        fCheckAlarms(frmAlarm, vHours, vMinutes);
			}
    }

    // loop function every 1000 milliseconds to simulate clock tick
    var vTimerID = setTimeout('fShowTime(' + argFrmClock.name + ');',1000);
  }
}

function fLocaleTime(argFrmLocation, argFrmTimeZones) {
  // compute a time output for the specified time zone; called by fShowTime once per minute
  with (document) {
    with (argFrmLocation) {
      var vZone = (Math.abs(frmLocationUTCOffsetInput.value) + (+frmLocationUTCAdjustInput.value));
      vZone     = fSign(frmLocationUTCOffsetInput.value, vZone); // preserve sign of offset
      var vDST  = +frmLocationDSTInput.value;
    }

    argFrmLocation.frmLocationLocalTimeDisplay.value = fGetTimeDisplay(vZone, vDST, vglobalLocaleStyle);

    with (argFrmTimeZones) {
      if (argFrmTimeZones.style.display == 'block') { // skip if the Time Zone form is not displayed
        for (var vIndex = 1; vIndex <= vglobalNumberTimeZoneRows; vIndex++) {
          eval ("var vOffset = frmTimeZoneOffset"+vIndex+".value");
          if (vOffset != '') {
              eval ("var vDST = (frmTimeZoneDST"+ vIndex +".checked ? 60 : 0)");
              eval ("frmTimeZoneTime"+ vIndex +".value = fGetTimeDisplay(vOffset, vDST, vglobalTZStyle)");
          } else {
              eval ("frmTimeZoneTime"+ vIndex +".value = ''");
          }
        }
      }
    }
  }
}

function fGetTimeDisplay(argZone, argDST, argDisplayStyle) {
  // get UTC time; add in the selected offset & adjustment to find the time in the selected time zone
  // adjust UTCTime for Time Zone & DST; ignore seconds;
  var vLocalTimeObj = new Date();
  with (vLocalTimeObj) {
    var vUTCTime   = 60*getUTCHours() + getUTCMinutes() + (60*argZone) + argDST; //in minutes
    var vTimeArray = fMinutesToHM(vUTCTime);
    var vHours     = vTimeArray[0];
    var vMinutes   = vTimeArray[1];

    var vDayChange = getUTCDate();

    if (vHours > 23) {
      vHours -= 24;
      vDayChange++;
    }
    if (vHours < 0) {
      vHours += 24;
      vDayChange--;
      if (vDayChange == 0) { // fix anomaly at month end if within local offset from UTC
        setUTCHours(-argZone);
      }
    }
    setUTCDate(vDayChange); // adjust date, since above adjustments could change the date in a time zone
    vHours %= 24            // adjust '24' to 'O'

    switch (+argDisplayStyle) {
      case 0:  // fix am/pm hours display to be 1-12, & midnight display to be 12 hours, not 0 hours
        var vDisplayHours  = (vHours>12) ? vHours-12 : (vHours==0) ? 12 : vHours;
        var vDisplayString = fHMTimeString(vDisplayHours, vMinutes, false)
                             +((vHours>=12) ? ' ' + msg_pm : ' ' + msg_am) + ' '
                             + getUTCDate() + ' ' + kShortMonthName[getUTCMonth()];
        break;
      case 1:  // 24 hr mode
        var vDisplayString = fHMTimeString(vHours, vMinutes, true) + ' '
                             + getUTCDate() +  ' ' + kShortMonthName[getUTCMonth()];
        break;
      case 2:  // UTC Time
        var vDisplayString = msg_utc + ' ' + fHMTimeString(getUTCHours(), getUTCMinutes(), true) + ' '
                             + getUTCDate() +  ' ' + kShortMonthName[getUTCMonth()];
        break;
    }
  }
  return (vDisplayString);
}

function fInternetBeatTime(argTimeObj) {
  // computer "Swatch" Internet beat time
  var vOffset = (argTimeObj.getTimezoneOffset() + 60) * 60 * 1000;  // in milliseconds
  var vBielTime = new Date(argTimeObj.getTime() + vOffset); //Biel, Switzerland time

  var vBHours_ms   = (vBielTime.getHours())   * 1000 * 60 * 60; // in milliseconds
  var vBMinutes_ms = (vBielTime.getMinutes()) * 1000 * 60;
  var vBSeconds_ms = (vBielTime.getSeconds()) * 1000;
  var vTotalMs     = vBHours_ms + vBMinutes_ms + vBSeconds_ms;

  var vBeats       = fZeroPad((Math.floor((vTotalMs) / (60*60*24))), 3);

  // divide the basic unit by 10 to get sub beats
  var vTimeX = Math.floor((vTotalMs) / (60*60*24/10)) / 10;
  var vSubBeats = Math.round((1-(Math.ceil(vTimeX)-vTimeX))*10);
  if (vSubBeats == 10) vSubBeats = '0';
  vSubBeats     = fZeroPad(vSubBeats, 1);

  return ('@' + vBeats + '.' + vSubBeats);
}

//==========================
// Date Arithmetic Functions
//==========================

function fUpdateDateCalculations (argForm) {
  with (document) {
    if (argForm.style.display == 'none') { return; } // skip if form isn't displayed

    fUpdateDateArithmetic(argForm);
    fUpdateDateDiffCalc(argForm);
    fUpdateDaysToGoCalc(argForm);
  }
}

function fDateAddSubtract(argOperation, argOperationName, argDayDelta, argDayDeltaName, argDaysFromDateObj) {
  //add or subtract specified days to a date picked from the calendar
  if (isNaN(argDayDelta)) {
    argDaysFromDateObj.value = '';
    alert (msgNaN);
    return (false);
  } else {
    fWriteINI(kSettingsFile, 'DateCalc', argOperationName, argOperation);
    fWriteINI(kSettingsFile, 'DateCalc', argDayDeltaName,  argDayDelta);
    
    if (argDayDelta == '') {
      argDaysFromDateObj.value = ''
      return (false);
    } else {
      // compute Julian day from selected date; then just add/subtract number
      var vJDay = fJDayFromYMD(vglobalDate.getFullYear(), vglobalDate.getMonth()+1, vglobalDate.getDate());
      switch (argOperation) {
        case '+' : vJDay += +argDayDelta; break;
        case '-' : vJDay -=  argDayDelta; break;
      }
      var vDateYMD = fYMDFromJDay(vJDay); // convert back to displayable format
      argDaysFromDateObj.value = fDateDisplay(vDateYMD, vglobalDateStyle);
    }
  }
}

function fUpdateDateArithmetic(argForm) {
  // Recompute ALL of the dates on which date arithmetic is being performed.
  //
  // The 'eval' function creates JS commands out of strings, allowing a loop
  // through the similarly named form fields; shorter and more general than old code.
  // fDateAddSubtract needs frmDateCalcDaysFromDate+vIndex to be an object
  with (document) {
    with (argForm) {
      for (var vIndex = 1; vIndex <= vglobalNumDateArithmeticRows; vIndex++) {
        eval ('fDateAddSubtract(frmDateCalcOperation' + vIndex + '.value,'+
              'frmDateCalcOperation' + vIndex + '.id, frmDateCalcDayDelta' + vIndex + '.value,'+
              'frmDateCalcDayDelta' + vIndex + '.id, frmDateCalcDaysFromDate' + vIndex + ')');
      }
    }
  }
}

function fCalcDateDiff(argJDateFirstValue, argJDateFirstName, argJDateSecondValue, argJDateSecondName, argDaysBetweenDatesObj) {
  // calculate the difference between 2 Julian dates and then display
  if ((argJDateFirstValue != 0) && argJDateSecondValue != 0) {
    argDaysBetweenDatesObj.value = Math.abs(argJDateFirstValue - argJDateSecondValue);
  } else {
    argDaysBetweenDatesObj.value = '';
  }
  fWriteINI(kSettingsFile, 'DateCalc', argJDateFirstName, argJDateFirstValue);
  fWriteINI(kSettingsFile, 'DateCalc', argJDateSecondName, argJDateSecondValue);
}

function fUpdateDateDiffCalc(argForm) {
  // Recompute ALL of the dates being counted down from, and number of days to go.
  // Each hidden Julian date value is read and used to compute the display fields.
  //
  // The 'eval' function creates JS commands out of strings, allowing a loop
  // through the similarly named form fields; shorter and more general than old code.
  with (document) {
    with (argForm) {
      for (var vIndex = 1; vIndex <= vglobalNumDateDiffRowPairs; vIndex++) {
        eval('var vJDay1 = frmJDateFirst'+vIndex+'.value');
        if (vJDay1 == '' || vJDay1 == 0) {  // error if hidden date is missing or zero
          vJDay1           = 0;
          var vJDateFirst  = ' ';
          var vChkBox1     = false;
        } else {  // if the hidden date is found, convert it to a readable style
          var vDateYMD     = fYMDFromJDay(+vJDay1);
          var vJDateFirst  = fDateDisplay(vDateYMD, vglobalDateStyle);
          var vChkBox1     = true;
        }
        eval('frmDateCalcDateSelectFirst'+vIndex+'.checked = vChkBox1');
        eval('frmDateCalcBetweenDatesFirst'+vIndex+'.value = vJDateFirst');
        eval('frmJDateFirst'+vIndex+'.value = vJDay1'); // hidden variable with JDay value

        eval('var vJDay2 = frmJDateSecond'+vIndex+'.value');
        if (vJDay2=='' || vJDay2==0) {  // some error checking, if the hidden date is missing or zero
          vJDay2           = 0;
          var vJDateSecond = ' ';
          var vChkBox2     = false;
        } else {  // if the hidden date is found, convert it to a readable style
          var vChkBox2     = true;
          var vDateYMD     = fYMDFromJDay(+vJDay2);
          var vJDateSecond = fDateDisplay(vDateYMD, vglobalDateStyle);
        }
        eval('frmDateCalcDateSelectSecond'+vIndex+'.checked = vChkBox2');
        eval('frmDateCalcBetweenDatesSecond'+vIndex+'.value = vJDateSecond');
        eval('frmJDateSecond'+vIndex+'.value = vJDay2'); // hidden variable with JDay value

        // fCalcDateDiff needs frmDateCalcDaysBetweenDates+vIndex to be an object
        eval('fCalcDateDiff(vJDay1, frmJDateFirst'+vIndex+'.id, vJDay2, frmJDateSecond'+vIndex+'.id, frmDateCalcDaysBetweenDates'+vIndex+')');
      }
    }
  }
}

function fCalcDaysToGo(argJDateValue, argJDateName, argDaysToGoObj) {
  // calculate the difference between a selected date (expressed as a Julian date), and
  // a single Julian date argument; save the Julian date to an INI file
  if (argJDateValue != 0) {
    //  var vToday          = new Date();
    //  var vJDayToday      = fJDayFromYMD(vToday.getFullYear(), vToday.getMonth()+1, vToday.getDate());
    var vJDayToday      = fJDayFromYMD(vglobalDate.getFullYear(), vglobalDate.getMonth()+1, vglobalDate.getDate());
    var vDateDifference = +argJDateValue - vJDayToday;
  } else {
    var vDateDifference = '';
  }
  argDaysToGoObj.value  = vDateDifference;
  fWriteINI(kSettingsFile, 'DateCalc', argJDateName, argJDateValue);
}

function fUpdateDaysToGoCalc(argForm) {
  // Recompute ALL of the dates being counted down from, and number of days to go.
  // Each hidden Julian date value is read and used to compute the display fields.
  //
  // The 'eval' function creates JS commands out of strings, allowing a loop
  // through the similarly named form fields; shorter and more general than old code.
  with (document) {
    with (argForm) {
      for (var vIndex=1; vIndex<=vglobalNumCountdownRows; vIndex++) {
        eval('var vJDay = frmJDateCntDn'+vIndex+'.value');
        if (vJDay=='' || vJDay==0) {  // some error checking, if the hidden date is missing or zero
          vJDay                = 0;
          var vDateToCountFrom = ' ';
          var vChkBox          = false;
        } else {  // if the hidden date is found, convert it to a readable style
          var vDateYMD         = fYMDFromJDay(+vJDay);
          var vDateToCountFrom = fDateDisplay(vDateYMD, vglobalDateStyle);
          var vChkBox          = true;
        }
        eval('frmDateCalcCountDownChkBox'+vIndex+'.checked   = vChkBox');
        eval('frmDateCalcDateToCountFrom'+vIndex+'.value = vDateToCountFrom');
        eval('frmJDateCntDn'+vIndex+'.value      = vJDay'); // hidden variable with JDay value

        // fCalcDaysToGo needs frmDateCalcDaysToGo+vIndex to be an object
        eval('fCalcDaysToGo(vJDay, frmJDateCntDn'+vIndex+'.id, frmDateCalcDaysToGo'+vIndex+')');
      }
    }
  }
}

function fDateDisplay(argDate, argStyle) { 
  // argDate is in YMD form
  switch (argStyle) {
    case 0: return (fLeadZero1(argDate[2]) +'-'+ fLeadZero1(argDate[1]) +'-'+ argDate[0]); break;
    case 1: return (argDate[0] +'-'+ fLeadZero1(argDate[1]) +'-'+ fLeadZero1(argDate[2])); break;
  }
}

function fSelectDate(argChkBox, argFrmDateDisplayObj, argFrmHiddenJDayObj) {
  // when checkbox is clicked, fill in display field with selected date, AND
  // compute the Julan date and save that date value in a hidden input field
  if (argChkBox.checked) {
    argFrmHiddenJDayObj.value  = fJDayFromYMD(vglobalDate.getFullYear(), vglobalDate.getMonth()+1, vglobalDate.getDate());
    var vDateYMD               = fYMDFromJDay(+argFrmHiddenJDayObj.value);
    argFrmDateDisplayObj.value = fDateDisplay(vDateYMD, vglobalDateStyle);
  } else {
    argFrmDateDisplayObj.value = '';
    argFrmHiddenJDayObj.value  = 0;
  }
}

function fBuildDateArithmeticRows() {
  // dynamically build the selected number of "days from date" arithmetic rows
  for (var vIndex=1; vIndex<=vglobalNumDateArithmeticRows; vIndex++) {
    document.write("\<tr align=\"left\">"+
      "\<td class=\"borderRight\">\<select id=\"frmDateCalcOperation" + vIndex + "\" class=\"Operation\" title=\"Specify addition or subtraction time operation\" " +
      "  onChange=\"fDateAddSubtract(this.form.frmDateCalcOperation" + vIndex + ".value, this.form.frmDateCalcOperation" + vIndex + ".id,"+
      "  this.form.frmDateCalcDayDelta" + vIndex + ".value, this.form.frmDateCalcDayDelta" + vIndex + ".id, this.form.frmDateCalcDaysFromDate" + vIndex + ");\">"+
      "  \<option value=\"+\">\+ \<option value=\"-\">\</select>\</td>"+
      "\<td class=\"borderCenter\"> \<input type=\"text\" class=\"SkyTimesText\" id=\"frmDateCalcDayDelta" + vIndex + "\" title=\"Days to add to/subtract from selected date\""+
      "  onChange=\"fDateAddSubtract(this.form.frmDateCalcOperation" + vIndex + ".value, this.form.frmDateCalcOperation" + vIndex + ".id,"+
      "  this.form.frmDateCalcDayDelta" + vIndex + ".value, this.form.frmDateCalcDayDelta" + vIndex + ".id, this.form.frmDateCalcDaysFromDate" + vIndex + ");\">\</td>"+
      "\<td class=\"borderCenter\">=\</td>"+
      "\<td class=\"borderLeft\">\<input type=\"text\" class=\"SkyTimesStatic\" id=\"frmDateCalcDaysFromDate" + vIndex + "\" "+
      "  title=\"Date from selected date\" readonly=\"readonly\">\</td>"+
      "\</tr>");
  }
}

function fBuildDateDifferenceRows() {
  // dynamically build the selected number of date difference arithmetic row pairs
  for (var vIndex=1; vIndex<=vglobalNumDateDiffRowPairs; vIndex++) {
    document.write("\<tr align=\"left\">"+
      "\<td class=\"borderRightBottom\">\<input type=\"checkbox\" class=\"Checkbox\" id=\"frmDateCalcDateSelectFirst" + vIndex + "\"" +
      "  title=\"Set/clear first date\" "+
      "  onClick = \"fSelectDate (this, this.form.frmDateCalcBetweenDatesFirst" + vIndex + ", this.form.frmJDateFirst" + vIndex + ");" +
      "  fCalcDateDiff(this.form.frmJDateFirst" + vIndex + ".value, this.form.frmJDateFirst" + vIndex + ".id,"+
      "  this.form.frmJDateSecond" + vIndex + ".value, this.form.frmJDateSecond" + vIndex + ".id, "+
      "  this.form.frmDateCalcDaysBetweenDates" + vIndex + ");\">\<input type=\"hidden\" id=\"frmJDateFirst" + vIndex + "\">\</td>"+
      "\<td class=\"borderCenterBottom\">\<input type=\"text\" class=\"SkyTimesStatic\" id=\"frmDateCalcBetweenDatesFirst" + vIndex + "\" "+
      "  title=\"First date\" readonly=\"readonly\">\</td>" +
      "\<td class=\"borderCenter\" rowspan=\"2\" id=\"frmDateCalcDiff" + vIndex + "\">diff:\</td>"+
      "\<td class=\"borderCenter\" rowspan=\"2\">\<input type=\"text\" class=\"SkyTimesStatic\" id=\"frmDateCalcDaysBetweenDates" + vIndex + "\" " +
      "  title=\"Days between selected dates\" readonly=\"readonly\">\</td>\</tr>"+
      "\<tr>"+
      "\<td class=\"borderRightTop\">\<input type=\"checkbox\" class=\"Checkbox\" id=\"frmDateCalcDateSelectSecond" + vIndex + "\" " +
      "  title=\"Set/clear second date\" "+
      "  onClick = \"fSelectDate (this, this.form.frmDateCalcBetweenDatesSecond" + vIndex + ", this.form.frmJDateSecond" + vIndex + "); " +
      "  fCalcDateDiff(this.form.frmJDateFirst" + vIndex + ".value, this.form.frmJDateFirst" + vIndex + ".id, "+
      "  this.form.frmJDateSecond" + vIndex + ".value, this.form.frmJDateSecond" + vIndex + ".id, "+
      "  this.form.frmDateCalcDaysBetweenDates" + vIndex + ");\"> \<input type=\"hidden\" id=\"frmJDateSecond"+ vIndex + "\">\</td>"+
      "\<td class=\"borderCenterTop\">\<input type=\"text\" class=\"SkyTimesStatic\" id=\"frmDateCalcBetweenDatesSecond" + vIndex + "\" "+
      "  title=\"Second date\" readonly=\"readonly\">\</td>"+
      "\</tr>");
  }
}

function fBuildCountdownRows() {
  // dynamically build the selected number of "count to date" rows
  for (var vIndex=1; vIndex<=vglobalNumCountdownRows; vIndex++) {
    document.write("\<tr align=\"left\">"+
      "\<td class=\"borderRight\">\<input type=\"checkbox\" class=\"Checkbox\" id=\"frmDateCalcCountDownChkBox" + vIndex + "\"" +
      "  title=\"Set/clear target date\" "+
      "  onClick = \"fSelectDate(this, this.form.frmDateCalcDateToCountFrom"+ vIndex + ", this.form.frmJDateCntDn" + vIndex + ");" +
      "  fCalcDaysToGo(this.form.frmJDateCntDn" + vIndex + ".value, this.form.frmJDateCntDn" + vIndex + ".id, "+
      "  this.form.frmDateCalcDaysToGo" + vIndex + ");\">\<input type=\"hidden\" id=\"frmJDateCntDn" + vIndex + "\"> \</td>" +
      "\<td class=\"borderCenter\">\<input type=\"text\" class=\"SkyTimesStatic\" id=\"frmDateCalcDateToCountFrom"+ vIndex + "\" "+
      "  title=\"Target date to count from\" readonly=\"readonly\">\</td>" +
      "\<td class=\"borderCenter\" id=\"frmDateCalcToGo" + vIndex + "\">days:\</td>"+
      "\<td class=\"borderLeft\">\<input type=\"text\" class=\"SkyTimesStatic\" id=\"frmDateCalcDaysToGo" + vIndex + "\" "+
      "  title=\"Days away from selected date\" readonly=\"readonly\">\</td>"+
      "\</tr>");
   }
}

//==========================================
// Sunrise, Solar Noon, and Sunset Functions
//==========================================

function fSunInfoUpdate(argLocationForm, argSunTimeForm) {
  with (document) {
    if (argSunTimeForm.style.display == 'none') {return;} // skip if the form isn't displayed

    // computes sunrise, sunset, solar noon, equation of time, solar declination, & various twilights
    with (argLocationForm) {
      // remember to add 1 to month so that January=1, not 0
      var vSunCalArray = fMainSunCalc(vglobalDate.getFullYear(), vglobalDate.getMonth()+1, vglobalDate.getDate(),
        frmLocationUTCOffsetInput.value, frmLocationUTCAdjustInput.value, frmLocationDSTInput.value,
        frmLatDegInput.value,            frmLatMinInput.value,            frmLatSecInput.value,          frmLocationLatDirInput.value,
        frmLongDegInput.value,           frmLongMinInput.value,           frmLongSecInput.value,         frmLocationLongDirInput.value,
        frmLocationHoursInput.value,     frmLocationMinutesInput.value,   frmLocationSecondsInput.value, frmLocationTimeOverride.checked,
        argLocationForm,                 kSunRefractionOffset,            vglobalLocTimeStyle);
    }
    with (argSunTimeForm) {
      frmSunEqOfTimeDisplay.value         = vSunCalArray[0];
      frmSunRightAscensionDisplay.value   = vSunCalArray[1];
      frmSunDeclinationDisplay.value      = vSunCalArray[2];
      frmSunElevationDisplay.value        = vSunCalArray[3];
      frmSunAzimuthDisplay.value          = vSunCalArray[4];
      frmSunDistanceDisplay.value         = vSunCalArray[5];
      frmSunriseDisplay.value             = vSunCalArray[6];
      frmSunSolarNoonDisplay.value        = vSunCalArray[7];
      frmSunsetDisplay.value              = vSunCalArray[8];
      frmSunDayLengthDisplay.value        = vSunCalArray[9];
      frmLabelSunExtraInfo.innerHTML      = msgSkyInfo +' '+ vSunCalArray[10] + '';
    }
    with (argLocationForm) {// calculate civil twilight
      // remember to add 1 to month so that January=1, not 0
      var vSunCalArray = fMainSunCalc(vglobalDate.getFullYear(), vglobalDate.getMonth()+1, vglobalDate.getDate(),
        frmLocationUTCOffsetInput.value, frmLocationUTCAdjustInput.value, frmLocationDSTInput.value,
        frmLatDegInput.value,            frmLatMinInput.value,            frmLatSecInput.value,          frmLocationLatDirInput.value,
        frmLongDegInput.value,           frmLongMinInput.value,           frmLongSecInput.value,         frmLocationLongDirInput.value,
        frmLocationHoursInput.value,     frmLocationMinutesInput.value,   frmLocationSecondsInput.value, frmLocationTimeOverride.checked,
        argLocationForm,                 kCivilTwilight,                  vglobalLocTimeStyle);
    }
    with (argSunTimeForm) {// display civil twilight
      frmSunTwilightCivilAMDisplay.value    = vSunCalArray[6];
      frmSunTwilightCivilPMDisplay.value    = vSunCalArray[8];
    }
    with (argLocationForm) {// calculate nautical twilight
      // remember to add 1 to month so that January=1, not 0
      var vSunCalArray = fMainSunCalc(vglobalDate.getFullYear(), vglobalDate.getMonth()+1, vglobalDate.getDate(),
        frmLocationUTCOffsetInput.value, frmLocationUTCAdjustInput.value, frmLocationDSTInput.value,
        frmLatDegInput.value,            frmLatMinInput.value,            frmLatSecInput.value,          frmLocationLatDirInput.value,
        frmLongDegInput.value,           frmLongMinInput.value,           frmLongSecInput.value,         frmLocationLongDirInput.value,
        frmLocationHoursInput.value,     frmLocationMinutesInput.value,   frmLocationSecondsInput.value, frmLocationTimeOverride.checked,
        argLocationForm,                 kNauticalTwilight,               vglobalLocTimeStyle);
    }
    with (argSunTimeForm) {// display nautical twilight
      frmSunTwilightNauticalAMDisplay.value = vSunCalArray[6];
      frmSunTwilightNauticalPMDisplay.value = vSunCalArray[8];
    }
    with (argLocationForm) {// calculate astronomical twilight
      // remember to add 1 to month so that January=1, not 0
      var vSunCalArray = fMainSunCalc(vglobalDate.getFullYear(), vglobalDate.getMonth()+1, vglobalDate.getDate(),
        frmLocationUTCOffsetInput.value, frmLocationUTCAdjustInput.value, frmLocationDSTInput.value,
        frmLatDegInput.value,            frmLatMinInput.value,            frmLatSecInput.value,          frmLocationLatDirInput.value,
        frmLongDegInput.value,           frmLongMinInput.value,           frmLongSecInput.value,         frmLocationLongDirInput.value,
        frmLocationHoursInput.value,     frmLocationMinutesInput.value,   frmLocationSecondsInput.value, frmLocationTimeOverride.checked,
        argLocationForm,                 kAstronomicalTwilight,           vglobalLocTimeStyle);
    }
    with (argSunTimeForm) {// display astronomical twilight
      frmSunTwilightAstroAMDisplay.value    = vSunCalArray[6];
      frmSunTwilightAstroPMDisplay.value    = vSunCalArray[8];
    }
  }
}

//===============================
// Moonrise and Moonset Functions
//===============================

function fMoonInfoUpdate(argLocationForm, argMoonTimeForm) {
  with (document) {
    if (argMoonTimeForm.style.display == 'none') {return;} // skip if the form isn't displayed

    // computes moonrise, moonset, position information, phase angle, & percentage of illumination
    with (argLocationForm) {// remember to add 1 to month so that January=1, not 0
      var vMoonCalArray = fMainMoonCalc(vglobalDate.getFullYear(), vglobalDate.getMonth()+1, vglobalDate.getDate(),
        frmLocationUTCOffsetInput.value, frmLocationUTCAdjustInput.value, frmLocationDSTInput.value,
        frmLatDegInput.value,            frmLatMinInput.value,            frmLatSecInput.value,          frmLocationLatDirInput.value,
        frmLongDegInput.value,           frmLongMinInput.value,           frmLongSecInput.value,         frmLocationLongDirInput.value,
        frmLocationHoursInput.value,     frmLocationMinutesInput.value,   frmLocationSecondsInput.value, frmLocationTimeOverride.checked,
        argLocationForm,                 vglobalLocTimeStyle);
    }
    with (argMoonTimeForm) {
      frmMoonRightAscensionDisplay.value = vMoonCalArray[0];
      frmMoonDeclinationDisplay.value    = vMoonCalArray[1];
      frmMoonElevationDisplay.value      = vMoonCalArray[2];
      frmMoonAzimuthDisplay.value        = vMoonCalArray[3];
      frmMoonDistanceDisplay.value       = vMoonCalArray[4];
      frmMoonPhaseDisplay.value          = vMoonCalArray[5];
//    frmMoonPhaseDisplay.value          = vMoonCalArray[5] + ' ' +vMoonCalArray[10];
      frmMoonLightDisplay.value          = vMoonCalArray[6];
      frmMoonriseDisplay.value           = vMoonCalArray[7];
      frmMoonsetDisplay.value            = vMoonCalArray[8];
      frmLabelMoonExtraInfo.innerHTML    = msgSkyInfo +' '+ vMoonCalArray[9] + '';
      frmMoonPhaseImage.src              = kImageFiles +'MoonPhaseImage'+vMoonCalArray[10]+'.gif';
    }
  }
}

//====================
// Time Zone Functions
//====================

function fBuildTimeZoneRows() {
  // dynamically build the selected number of "days from date" arithmetic rows
  for (var vIndex=1; vIndex<=vglobalNumberTimeZoneRows; vIndex++) {
    document.write("\<tr>"+
      "\<td class=\"borderRight\">\<input type=\"checkbox\" class=\"Checkbox\" id=\"frmTimeZone"+vIndex+"\""+
      "  title=\"Set/clear time for selected city\" "+
      "  onClick=\"fGetLocationData(this, this.form.frmTimeZoneCountry, this.form.frmTimeZoneCity,"+
      "  this.form.frmTimeZoneLocation"+vIndex+", this.form.frmTimeZoneDST"+vIndex+", this.form.frmTimeZoneOffset"+vIndex+","+
      "  this.form.frmTimeZoneTime"+vIndex+", frmLocation); "+
      "  fLocaleTime(frmLocation, frmTimeZones);\">\</td>"+
      "\<td class=\"borderLeft\">\<input type=\"checkbox\" class=\"Checkbox\" id=\"frmTimeZoneDST"+vIndex+"\""+
      "  title=\"Set/clear daylight saving time (Summer Time)\" "+
      "  onClick=\"fGetDSTData(this, this.form.frmTimeZoneOffset"+vIndex+", "+
      "  this.form.frmTimeZoneSelectFlag,this.form.frmTimeZoneLocation"+vIndex+", frmLocation);"+
      "  fLocaleTime(frmLocation, frmTimeZones);"+
      "  fSunInfoUpdate(frmLocation, frmSunTimes);"+
      "  fMoonInfoUpdate(frmLocation, frmMoonTimes);\">\</td>"+
      "\<td>\<input type=\"text\" class=\"TimeZoneLoc\" "+
      "  title=\"City or location\" id=\"frmTimeZoneLocation"+vIndex+"\">\</td>"+
      "\<td>\<input type=\"hidden\" id=\"frmTimeZoneOffset"+vIndex+"\">"+
      "  \<input type=\"text\" class=\"TimeZoneInfoRt\" id=\"frmTimeZoneTime"+vIndex+"\" readonly=\"readonly\""+
      "  title=\"Click to show coordinates and Sun/Moon times for this location\" "+
      "  onClick=\"fSetLocationData(this, this.form.frmTimeZoneLocation"+vIndex+","+
      "  this.form.frmTimeZoneDST"+vIndex+", this.form.frmTimeZoneSelectFlag, frmLocation);"+
      "  fLocaleTime(frmLocation, frmTimeZones);\">\</td>\</tr>");
  }
}

function fLoadTZCountries(argForm) {
  // read INI file of country and city data, until "Country" name defaults to "None"
  var vParamArray;
  argForm.frmTimeZoneCountry.options.length = 1 // zero out Country list except for default 1st item
  with (document) {
    for (var vIndex = 1; vIndex; vIndex++) { // Loop until no more Country names to read
      vParamArray = fReadINI(kCountriesFile, 'Countries', String(vIndex), 'None').split(',');
      if (vParamArray[0] != 'None') {
        argForm.frmTimeZoneCountry[vIndex] = new Option (text=vParamArray[0], value=vParamArray[1]);
      } else {
        break;
      }
    }
    // if a Last Country selected value exists, set the country display to that value
    // and load the corresponding cities
    var vSelected = fReadINI(kSettingsFile, 'TimeZoneLocations', 'LastCountry', '1');
    vSelected = (vSelected == '') ? 0 : +vSelected;
    argForm.frmTimeZoneCountry.value = vSelected;
    if (vSelected != 0) {fLoadTZCities(vSelected, frmTimeZones)};
  }
}

function fLoadTZCities(argValue, argForm) {
  // when a Country is selected, read INI file to get corresponding cities, until "City" name defaults to "None"
  fWriteINI(kSettingsFile, 'TimeZoneLocations', 'LastCountry', argValue);  // save selected Country
  var vCountryID = fReadINI(kCountriesFile, 'Countries', String(argValue), 'None').split(',');
  var vLocationFile = kLocationsFile + String(vCountryID[2]) +'.ini';

  var vParamArray;
  with (document) {
    argForm.frmTimeZoneCity.options.length = 0;    // zero out any existing City list
    for (var vIndex = 1; vIndex; vIndex++) {       // Loop until no more City names to read
      vParamArray = fReadINI(vLocationFile, vCountryID[1], String(vIndex), 'None').split(',');
      if (vParamArray[0] != 'None') {
        if (vParamArray[0] == 'United States') { // for U.S., add in State abbreviations after City name
  	      argForm.frmTimeZoneCity[vIndex-1] = new Option (text=vParamArray[2] +' '+ vParamArray[1], value=vIndex);
        } else {
 	        argForm.frmTimeZoneCity[vIndex-1] = new Option (text=vParamArray[2], value=vIndex);
        }
      } else {
        break;
      }
    }
  }
}

function fGetLocationData(argChkBox, argCountry, argCity, argTZLocation, argDSTChkBox, argTZOffset, argTimeDisplay, argFrmLocation) {
  with (document) {
    if (argChkBox.checked) {  // checked box means get location data & load form
      switch (argCountry.value) {
        case '0' :  // if Custom is selected, copy Lat & Long from Set Location fields & store; Location name is from TZ form
          with (argFrmLocation) {
            var vLocationArray = ',,' + argTZLocation.value +','
    			                     + frmLatDegInput.value    +','+ frmLatMinInput.value     +','
			                         + frmLatSecInput.value    +','+ (frmLocationLatDirInput.value  == 0 ? 'N' : 'S') +','
			                         + frmLongDegInput.value   +','+ frmLongMinInput.value    +','
			                         + frmLongSecInput.value   +','+ (frmLocationLongDirInput.value == 0 ? 'W' : 'E') +','
			                         + frmLocationUTCOffsetInput.value +','+ frmLocationUTCAdjustInput.value;
            argDSTChkBox.checked = (frmLocationDSTInput.value == 0 ? false: true);
          }
          fWriteINI(kSettingsFile, 'TimeZoneLocations', argTZLocation.id, vLocationArray);
          fWriteINI(kSettingsFile, 'TimeZoneLocations', argDSTChkBox.id, argDSTChkBox.checked);
          // read the data after writing it; fixes some variable type problems
          vLocationArray      = fReadINI(kSettingsFile, 'TimeZoneLocations', argTZLocation.id, 'None').split(',');
          argTZLocation.value = vLocationArray[2];
          var vOffset         = (Math.abs(vLocationArray[11]) + (+vLocationArray[12]));
          argTZOffset.value   = fSign(vLocationArray[11], vOffset);
          break;

        default:  // read Lat & Long from Locations.ini, set display & hidden values
          var vCountryID = fReadINI(kCountriesFile, 'Countries', argCountry.value, 'None').split(',');
          var vLocationFile = kLocationsFile + String(vCountryID[2]) +'.ini';

          var vLocationArray    = fReadINI(vLocationFile, argCountry.value, argCity.value, 'None').split(',');
          if (vLocationArray[0]=='United States') {// for U.S., add in State abbreviations after City name
	          argTZLocation.value = vLocationArray[2] +' '+ vLocationArray[1];
	        } else {
	          argTZLocation.value = vLocationArray[2];
	        }
          var vOffset           = (Math.abs(vLocationArray[11]) + (+vLocationArray[12]));
          argTZOffset.value     = fSign (vLocationArray[11], vOffset);
          fWriteINI(kSettingsFile, 'TimeZoneLocations', argTZLocation.id, vLocationArray);
          fWriteINI(kSettingsFile, 'TimeZoneLocations', argDSTChkBox.id,  argDSTChkBox.checked);
      }
    } else { // clear display if unchecked
        argTZLocation.value  = '';
        argTZOffset.value    = '';
        argTimeDisplay.value = '';
        argDSTChkBox.checked = false;
        fWriteINI(kSettingsFile, 'TimeZoneLocations', argTZLocation.id, '');
        fWriteINI(kSettingsFile, 'TimeZoneLocations', argDSTChkBox.id,  '');
        fColorSelectedField('', frmTimeZones, 'frmTimeZoneTime', vglobalNumberTimeZoneRows);
      }
  }
}

function fGetDSTData(argDSTChkBox, argTZOffset, argFlag, argTZLocation, argFrmLocation) {
  // process DST checkbox; if checked, store state in Settings.ini with corresponding location
  fWriteINI(kSettingsFile, 'TimeZoneLocations', argDSTChkBox.id, argDSTChkBox.checked);// save checked state

  // if the current selection has been selected for postion display, make the DST settings correspond
  if (argFlag.value == argTZLocation.id) {
    argFrmLocation.frmLocationDSTInput.value = argDSTChkBox.checked ? 60 : 0;
  }
}

function fSetLocationData(argField, argTZLocation, argTZDST, argFlag, argFrmLocation) {
  // if a Time Zone time is selected, copy the corresponding data to the Set Location form,
  // & update Sun and Moon times
  with (document) {
    var vLocationArray = fReadINI(kSettingsFile, 'TimeZoneLocations', argTZLocation.id, 'None').split(',');
    var vDST           = fReadINI(kSettingsFile, 'TimeZoneLocations', argTZDST.id,      'false');
    if (vLocationArray != '') {
      with (argFrmLocation) {
        frmLatDegInput.value            = vLocationArray[3];
        frmLatMinInput.value            = vLocationArray[4];
        frmLatSecInput.value            = vLocationArray[5];
        frmLocationLatDirInput.value    = (vLocationArray[6] == 'N'? 0 : 1);
        frmLongDegInput.value           = vLocationArray[7];
        frmLongMinInput.value           = vLocationArray[8];
        frmLongSecInput.value           = vLocationArray[9];
        frmLocationLongDirInput.value   = (vLocationArray[10] == 'W'? 0 : 1);
        frmLocationUTCOffsetInput.value = vLocationArray[11];
        frmLocationUTCAdjustInput.value = vLocationArray[12];
        frmLocationDSTInput.value       = (vDST == 'false'? 0 : 60);
      }
    argFlag.value=argTZLocation.id;  // set a flag to indicate which TZ location is being displayed
    fColorSelectedField(argField, frmTimeZones, 'frmTimeZoneTime', vglobalNumberTimeZoneRows); // highlight the displayed TZ location
    fSunInfoUpdate(frmLocation, frmSunTimes);
    fMoonInfoUpdate(frmLocation, frmMoonTimes);
    }
  }
}

function fClearTZHiddenField(argForm) {
  // when Loading the default Location, clear the hidden field that indicated
  // that a time zone location had been selected for display in the location form
  argForm.frmTimeZoneSelectFlag.value = '';
}

function fColorSelectedField(argSelection, argForm, argFieldName, argTotalRows) {
  // set background color of selected field to specified highlight value;
  // clear any previously set background highlight;
  // if no selection specified, just clear all background highlights
  with (document) {
    for (var vIndex = 1; vIndex <= argTotalRows; vIndex++) {
      argForm[argFieldName + vIndex].style.backgroundColor = kNormalDayBgColor;
    }
    if (argSelection != '') argForm[argSelection.id].style.backgroundColor = kSelectedDayBgColor;
  }
}

//================
// Alarm Functions
//================

function fBuildAlarmTimeRows() {
  // dynamically build the selected number of alarm timer rows
  for (var vIndex=1; vIndex<=vglobalNumberAlarmRows; vIndex++) {
    document.write("\<tr>"+
    "\<td class=\"borderRight\">\<input type=\"checkbox\" class=\"Checkbox\" id=\"frmAlarmSet"+vIndex+"\" title=\"Set/clear alarm time\" "+
    "   onClick=\"fSetAlarm(this, this.form, this.form.frmAlarmOn"+vIndex+", this.form.frmAlarmTypeDisplay"+vIndex+", "+
    "   this.form.frmAlarmSnoozeTypeDisplay"+vIndex+", this.form.frmCounterFlag"+vIndex+", this.form.frmAlarmSnoozeTime"+vIndex+", "+
    "   this.form.frmAlarmTimeSet"+vIndex+", this.form.frmResidualSeconds"+vIndex+", this.form.frmAlarmTime"+vIndex+", this.form.frmAlarmRang"+vIndex+");\">\</td>"+
    "\<td class=\"borderLeft\">\<input type=\"checkbox\" class=\"Checkbox\" id=\"frmAlarmOn"+vIndex+"\" title=\"Alarm on/off\" "+
    "   onClick=\"fAlarmToggle(this, this.form, this.form.frmCounterFlag"+vIndex+", this.form.frmAlarmTime"+vIndex+", "+
    "   this.form.frmAlarmTimeSet"+vIndex+", this.form.frmResidualSeconds"+vIndex+", this.form.frmAlarmRang"+vIndex+");\">\</td>"+
    "\<td>\<img border=\"0\" id=\"frmAlarmTypeDisplay"+vIndex+"\" title=\"Clock icon means alarm time of day, Hourglass icon means countdown time\" "+
    "   alt=\"Clock icon means alarm time of day, Hourglass icon means countdown time\" src=\"images/Blank15x16.gif\" "+
    "   onClick=\"fDisplayAlarmDetails(document.frmAlarm.frmAlarmTime"+vIndex+");\"> "+
    "   \<img border=\"0\" id=\"frmAlarmSnoozeTypeDisplay"+vIndex+"\" title=\"Recycle icon means snooze for set time\" "+
    "   alt=\"Recycle icon means snooze for set time\" src=\"images/Blank15x16.gif\"  "+
    "   onClick=\"fDisplayAlarmDetails(document.frmAlarm.frmAlarmTime"+vIndex+");\">\</td>"+
    "\<td class=\"Right\">\<input type=\"text\" class=\"TimeZoneInfoRt\" id=\"frmAlarmTime"+vIndex+"\" title=\"Alarm time setting\" "+
    "   onClick=\"fStopAlarm(this.form.frmAlarmTime"+vIndex+", this.form.frmAlarmRang"+vIndex+"); "+
    "   fSnooze(this.form, this.form.frmCounterFlag"+vIndex+", this.form.frmAlarmSnoozeTime"+vIndex+", "+
    "   this.form.frmAlarmTime"+vIndex+", this.form.frmAlarmTimeSet"+vIndex+", this.form.frmResidualSeconds"+vIndex+", this.form.frmAlarmRang"+vIndex+"); "+
    "   fCopyValuesToForm(this.form, this.form.frmCounterFlag"+vIndex+", this.form.frmAlarmTime"+vIndex+", this.form.frmAlarmRang"+vIndex+");\"> "+
    "   \<input type=\"hidden\" id=\"frmResidualSeconds"+vIndex+"\"> "+
    "   \<input type=\"hidden\" id=\"frmCounterFlag"+vIndex+"\"> "+
    "   \<input type=\"hidden\" id=\"frmAlarmTimeSet"+vIndex+"\"> "+
    "   \<input type=\"hidden\" id=\"frmAlarmSnoozeTime"+vIndex+"\"> "+
    "   \<input type=\"hidden\" id=\"frmAlarmRang"+vIndex+"\">\</td> "+
    "\</tr>");
  }
}

function fCheckAlarms(argForm, argHours, argMinutes) {
  if (vglobalAlarmLoop) {  // do this ONLY once per minute
    vglobalAlarmLoop = false;  // deactivate function until reactivated by setTimeout below
    with (document) {
      with (argForm) {
        for (var vIndex = 1; vIndex <= vglobalNumberAlarmRows; vIndex++) {
          eval("var vOnFlag = frmAlarmOn"+vIndex+".checked;");
          if (vOnFlag) { // do this part only if alarm is on
            eval("var vCounterFlag = +frmCounterFlag"+vIndex+".value;");
            switch (vCounterFlag) {
              case 0 :  // for absolute time
                eval("var vAlarmTime = frmAlarmTimeSet"+vIndex+".value;");
                if ((60*argHours + (+argMinutes)) == vAlarmTime) {
                  eval("frmAlarmRang"+vIndex+".value = 1;");
                  eval("fSoundAlarm(frmAlarmTime"+vIndex+");");
                }
                break;

              case 1 : // for relative (elapsed or countdown) time; once at 0, delay sounding alarm for ResidualSeconds, for better accuracy
                eval("frmAlarmTimeSet"+vIndex+".value--;");  // decrement countdown time
                eval("var vTime = fMinutesToHM(frmAlarmTimeSet"+vIndex+".value);");
                eval("var vAlarmTimeSetValue = +frmAlarmTimeSet"+vIndex+".value;");
                if (vAlarmTimeSetValue == 0) {
                  eval("frmAlarmRang"+vIndex+".value = 1;");
                  eval("var vDelay = frmResidualSeconds"+vIndex+".value;");
                  eval("var vID    = frmAlarmTime"+vIndex+".id;");

                  // following displays the residual seconds, counting down every 5 seconds
                  // after getting to a multiple of 5
                  fSetResidualTime(vIndex, vDelay); // show the original residual time

                  var vSeconds = vDelay % 5;  // get the "stub" time over a multiple of 5
                  vBaseDelay   = vDelay - vSeconds;
                  setTimeout("fSetResidualTime(" + vIndex + "," + vBaseDelay + ");", vSeconds*1000);

                  // set a series of commands to decrement the displayed residual time every 5 seconds
                  for (var vTime = vBaseDelay - 5; vTime >= 0; vTime -= 5) {
                    setTimeout("fSetResidualTime(" + vIndex + "," + vTime + ");", (vDelay-vTime)*1000);
                  }

                  // sound the alarm after the residual delay elapses
                  setTimeout("fSoundAlarm(document.getElementById('" + vID + "'))", vDelay*1000);
                } else if (vAlarmTimeSetValue >= 0) {
                  eval("frmAlarmTime"+vIndex+".value = fHMTimeString(vTime[0], vTime[1], true);");
                }
                break;
             }
          }
        }
      }
    }
  setTimeout('vglobalAlarmLoop = true',2000);  // reactivate function after 2 seconds
  }
}

function fSetResidualTime(argID, argTime) {
  // Update an alarm display to show  residual seconds
  eval("document.frmAlarm.frmAlarmTime"+argID+".value = '00:00:' + fLeadZero1(" + argTime + ");");
}

function fAlarmToggle(argChkBox, argForm, argCounterFlag, argAlarmTime, argAlarmTimeSet, argResidualSeconds, argAlarmRang) {
  // toggle Alarm on or off; save state in INI file
  with (document) {
    with (argForm) {
      argAlarmRang.value = 0;

      var vSettingsArray = fReadINI(kSettingsFile, 'Alarms', argAlarmTime.id, '').split(',');
      if (vSettingsArray != '') {
        if (argChkBox.checked == false) { // if turning alarm off, stop any sound & reset some things
          fStopSound();
          fHighlightOff(argAlarmTime);
          vSettingsArray[0]   = false;  // alarm off
        } else {
          vSettingsArray[0]   = true;   // alarm on
          var vNow            = new Date();
          argResidualSeconds.value = vNow.getSeconds();
        }
        fWriteINI(kSettingsFile, 'Alarms', argAlarmTime.id, vSettingsArray);
      }
      fResetAlarmTime(argAlarmTime, argAlarmTimeSet);
    }
  }
}

function fStopAlarm(argAlarmTime, argAlarmRang) {
  // stop alarm from sounding or message displaying IF alarm has rang
  if (+argAlarmRang.value == 1) {
    fStopSound();
    fHighlightOff(argAlarmTime);
  }
}

function fSoundAlarm(argAlarmTime) {
  // read INI file for settings, sound alarm & display message if one exists
  var vSettingsArray = fReadINI(kSettingsFile, 'Alarms', argAlarmTime.id, '').split(',');
  if (vSettingsArray != '') {
    if (vSettingsArray[0] == 'true') { // do this only if alarm is still "on"
      fPlaySound(vSettingsArray[7], 'infinite');
      fHighlightOn(argAlarmTime);
      if (vSettingsArray[8] != '') alert(vSettingsArray[8]);
      if (+vSettingsArray[1] == 1) {argAlarmTime.value = '00:00';} // set display for counter to zero
    } else {  //  do this if alarm is off
      if (+vSettingsArray[1] == 1) { // reset display for counter
        argAlarmTime.value = fHMTimeString(vSettingsArray[3], vSettingsArray[4], true);
      }
    }
  }
}

function fSnooze(argForm, argCounterFlag, argSnoozeTime, argAlarmTime,
                 argAlarmTimeSet, argResidualSeconds, argAlarmRang) {
  // if alarm rang but snooze clicked, reset some things so alarm will sound again
  with (argForm) {
    if (+argAlarmRang.value == 1) {
      argAlarmRang.value    = 0;
      var vNow              = new Date();
      
      if (+argCounterFlag.value == 0 && +argSnoozeTime.value != 0) { // for absolute alarm IF Snooze Delay is set
        var vNewTime          = vNow.getHours()*60 + vNow.getMinutes() + (+argSnoozeTime.value);
        argAlarmTimeSet.value = vNewTime;
        var vTime             = fMinutesToHM(vNewTime);
        argAlarmTime.value    = fHMFormat(vTime[0], vTime[1]);
      }
      
      if (+argCounterFlag.value == 1) { // only for countdown alarm
        argResidualSeconds.value = vNow.getSeconds();
        var vNewTime             = +argAlarmTimeSet.value + (+argSnoozeTime.value);
        argAlarmTimeSet.value    = vNewTime;
        var vTime                = fMinutesToHM(vNewTime);
        argAlarmTime.value       = fHMTimeString(vTime[0], vTime[1], true);
      }
    }
  }
}

function fPlayAlarmSelection(argAlarmSoundSelection) {
  // play the selected alarm sound
  with (document) {
    if (argAlarmSoundSelection == 'none') return; // skip if no sound to play
    if (document.all.AlarmSound.src == '') {
      frmAlarm.frmAlarmSoundControl.src = kImageFiles + 'imageStop.gif';
      fPlaySound(argAlarmSoundSelection, 'infinite');
    } else {
      frmAlarm.frmAlarmSoundControl.src = kImageFiles + 'imagePlay.gif';
      fStopSound();
    }
  }
}

function fPlaySound(argAlarmSound, argDuration) {
  document.all.AlarmSound.loop = argDuration;
  document.all.AlarmSound.src  = kSoundFiles + argAlarmSound;
}

function fStopSound() {
  document.all.AlarmSound.loop = 0;
  document.all.AlarmSound.src  = '';
}

function fSetAlarm(argSetCB, argForm, argAlarmOnCB, argAlarmIcon, argSnoozeIcon, argCounterFlag, 
                   argSnoozeTime, argAlarmTimeSet, argResidualSeconds, argAlarmTime, argAlarmRang) {
  // on checking Set, get alarm values and store for the selected alarm
  with (document) {
    with (argForm) {
      if (argSetCB.checked) { // checked means get & save settings
        argAlarmOnCB.checked     = true;
        argCounterFlag.value     = frmAlarmTypeSet.checked ? 1 : 0;

        argSnoozeTime.value      = frmAlarmSnoozeHoursInput.value*60.0 + (+frmAlarmSnoozeMinutesInput.value); // in minutes
        if ((argSnoozeTime.value == 0) && frmAlarmTypeSet.checked) { // set default snooze time for counters
          argSnoozeTime.value    = frmAlarmHoursInput.value*60.0 + (+frmAlarmMinutesInput.value); // in minutes
        }
        var vSnoozeFlag          = (argSnoozeTime.value == 0) ? 0 : 1;  // set flag if snooze delay not equal to zero

        if (frmAlarmTypeSet.checked) { // don't convert format for counters
          argAlarmTime.value     = fHMTimeString(frmAlarmHoursInput.value, frmAlarmMinutesInput.value, true);
        } else {
          argAlarmTime.value     = fHMFormat(frmAlarmHoursInput.value, frmAlarmMinutesInput.value);
        }

        argAlarmTimeSet.value    = frmAlarmHoursInput.value*60.0 + (+frmAlarmMinutesInput.value);
        var vNow = new Date();
        argResidualSeconds.value = vNow.getSeconds();

        argAlarmIcon.src         = kImageFiles + (frmAlarmTypeSet.checked ? 'HourGlass.gif' : 'AlarmClock.gif');
        argSnoozeIcon.src        = kImageFiles + (vSnoozeFlag == 1        ? 'Recycle.gif'   : 'Blank15x16.gif');

        var vSettingsArray       = argAlarmOnCB.checked             +','+ argCounterFlag.value        +',,'
                                   + frmAlarmHoursInput.value       +','+ frmAlarmMinutesInput.value  +','
                                   + frmAlarmSnoozeHoursInput.value +','+ frmAlarmSnoozeMinutesInput.value +','
                                   + frmAlarmSound.value            +','+ frmAlarmMessage.value;

        fWriteINI(kSettingsFile, 'Alarms', argAlarmTime.id, vSettingsArray);
        fWriteINI(kSettingsFile, 'Alarms', 'LastSelectedSound', frmAlarmSound.value);  // save default sound

      } else { // clear display if unchecked
        fStopSound();
        fHighlightOff(argAlarmTime);
        argAlarmOnCB.checked     = false;
        argCounterFlag.value     = 0;
        argSnoozeTime.value      = 0;
        argAlarmTime.value       = '';
        argAlarmTimeSet.value    = 0;
        argResidualSeconds.value = 0;
        argAlarmIcon.src         = kImageFiles + 'Blank15x16.gif';
        argSnoozeIcon.src        = kImageFiles + 'Blank15x16.gif';
        argAlarmRang.value       = 0;

        fWriteINI(kSettingsFile, 'Alarms', argAlarmTime.id, '');
      }
    }
  }
}

function fResetAlarmTime(argAlarmTime, argAlarmTimeSet) {
  // reset selected alarm to saved settings
  var vSettingsArray = fReadINI(kSettingsFile, 'Alarms', argAlarmTime.id, '').split(',');
  if (vSettingsArray != '') {
    if (vSettingsArray[1] == '0') { 
      argAlarmTime.value       = fHMFormat(vSettingsArray[3], vSettingsArray[4]);
    } else { // don't convert format for counters
      argAlarmTime.value       = fHMTimeString(vSettingsArray[3], vSettingsArray[4], true);
    }
    argAlarmTimeSet.value = vSettingsArray[3]*60.0 + (+vSettingsArray[4]);
  }
}

function fLoadAlarmSounds(argForm) {
  // read INI file for alarm .WAV file names, until "Alarm" name defaults to null string
  with (document) {
    var vDisplayName = '';
    with (document) {
      var vSoundFileName;
      for (var vIndex = 1; vIndex; vIndex++) { // Loop until no more alarm names to read
        vSoundFileName = fReadINI(kSettingsFile, 'AlarmSounds', String(vIndex), '');
        if (vSoundFileName != '') {
          vDisplayName = vSoundFileName.substr(0, vSoundFileName.indexOf('.'));
          argForm.frmAlarmSound[vIndex] = new Option (text=vDisplayName, value=vSoundFileName);
        } else {
          break;
        }
      }
    }
  }
}

function fReInitAlarmTimes(argForm) {
  // reload Alarm times when the time format for the main clock changes from AM/PM to 24Hr
  with (document) {
    for (var vIndex = 1; vIndex <= vglobalNumberAlarmRows; vIndex++) {
      eval("var vSettingsArray = fReadINI(kSettingsFile, 'Alarms', argForm.frmAlarmTime"+vIndex+".id, '').split(',');");
      if (vSettingsArray != '') {
        if (vSettingsArray[1] == '0') { // don't convert format for counters
          eval("argForm.frmAlarmTime"+vIndex+".value = fHMFormat(vSettingsArray[3], vSettingsArray[4]);");
        } else {
          eval("argForm.frmAlarmTime"+vIndex+".value = fHMTimeString(vSettingsArray[3], vSettingsArray[4], true);");
        }
      }
    }
  }
}

function fHighlightOn(argObject) {
  // highlight selected object
  argObject.style.backgroundColor = kSelectedDayBgColor;
}

function fHighlightOff(argObject) {
  // de-highlight selected object
  argObject.style.backgroundColor = kNormalDayBgColor;
}

function fCopyValuesToForm(argForm, argCounterFlag, argAlarmTime, argAlarmRang) {
  // when a setting is clicked, copy its values to the Alarm Setup form for display
  with (argForm) {
    if (+argAlarmRang.value == 0) { // enable only if this alarm has not rung
      var vSettingsArray = fReadINI(kSettingsFile, 'Alarms', argAlarmTime.id, '').split(',');
      if (vSettingsArray != '') {
        frmAlarmTypeSet.checked          = +argCounterFlag.value;
        frmAlarmHoursInput.value         = vSettingsArray[3];
        frmAlarmMinutesInput.value       = vSettingsArray[4];
        frmAlarmSnoozeHoursInput.value   = vSettingsArray[5];
        frmAlarmSnoozeMinutesInput.value = vSettingsArray[6];
        frmAlarmSound.value              = vSettingsArray[7];
        frmAlarmMessage.value            = vSettingsArray[8];
      }
    }
  }
}

function fDisplayAlarmDetails(argAlarmTime) {
  // show Alert with Snooze time, alarm sound name, & alarm message
  var vDisplayMsg = '';
  with (document) {
    var vSettingsArray = fReadINI(kSettingsFile, 'Alarms', argAlarmTime.id, '').split(',');
    if (vSettingsArray != '') {
      if (vSettingsArray[1] == '0') { // type is absolute alarm
        if (vSettingsArray[5] == '0' && vSettingsArray[6] == '0') { // no snooze time set
          var vSnooze = msgNoSnoozeSet;
        } else { // get snooze time
          var vSnooze = msgAlarmSnoozes + ' ' + fHMTimeString(vSettingsArray[5], vSettingsArray[6], false);
        }
      } else {   // type is counter
        if (vSettingsArray[5] == '0' && vSettingsArray[6] == '0') { // no snooze time set
          var vSnooze = msgCounterRepeats + ' ' + fHMTimeString(vSettingsArray[3], vSettingsArray[4], false);
        } else { // get snooze time
          var vSnooze = msgCounterRepeats + ' ' + fHMTimeString(vSettingsArray[5], vSettingsArray[6], false);
        }
      }
      var vMessage  = vSettingsArray[8]=='' ? msgNoMessage : vSettingsArray[8];
      vDisplayMsg   = vSnooze +'\n\n' + msgAlarmSound + ' ' + vSettingsArray[7] +'\n\n' + msgMessage + ' ' + vMessage;
    } else {
      vDisplayMsg   = msgNoAlarmSettings;
    }
    alert(vDisplayMsg);
  }
}

//===================
// Location Functions
//===================

function fUseLocalTime(argForm) {
  // read the local PC to find out the offset from GMT/UTC
  var vNow           = new Date();
  var vOffsetMinutes = Math.abs(vNow.getTimezoneOffset());
  var vSign          = (vNow.getTimezoneOffset() < 0 ? 1 : -1); // reverse sign
  var vOffsetHours   = vSign * Math.floor(vOffsetMinutes/60);
  var vAdjust        = '';

  switch (vOffsetMinutes%60) {// set the display to the necessary offset adjustment fraction
    case  0: vAdjust= '0.00'; break;
    case 15: vAdjust= '0.25'; break;
    case 30: vAdjust= '0.50'; break;
    case 45: vAdjust= '0.75'; break;
  }

  with (argForm) {
    frmLocationUTCOffsetInput.value = vOffsetHours;
    frmLocationUTCAdjustInput.value = vAdjust;
    frmLocationDSTInput.value       = 0; // no way in Javascript to determine if DST is set
  }
}

function fTZCheck(argForm) {
  // make sure Time Zone sign corresponds to Longitude
  var vCheck = 0;
  with (argForm) {
    switch (+frmLocationLongDirInput.value) {
      case 0 : if (+frmLocationUTCOffsetInput.value > 0) vCheck = 1; break; // W long., -TZ
      case 1 : if (+frmLocationUTCOffsetInput.value < 0) vCheck = 1; break; // E long., +TZ
    }
  }
  frmTimeZoneError.style.display = (vCheck == 0 ?'none' : 'block');
}

//==================
// Utility functions
//==================

function fSetTextbox(argTextBox, argStyle, argAlign, argValue) {
  with (argTextBox) {
    style.fontStyle = argStyle;
    style.textAlign = argAlign;
    value = argValue;
  }
}

function fCycleStyle(argStyle, argSection, argSequenceLength) {
  // cycle through styles, modulo argSequenceLength, and save to INI file
  var vStyle = (+argStyle+1)%argSequenceLength;
  fWriteINI(kSettingsFile, 'AppStyle', argSection, vStyle);
  return (vStyle);
}

function fToggleLabel(argForm, argFormFieldLabel, argClosedText, argOpenText) {
  // determine display style for form, and set labels with Closed or Open text
  with (document) {
    argFormFieldLabel.value = (argForm.style.display == 'none' ? argClosedText : argOpenText);
  }
}

function fHideForms() {
  // dynamically process a list of forms to hide them
  for (var vIndex = 0; vIndex < arguments.length; vIndex++) {
    document.all[arguments[vIndex].name].style.display = 'none';
  }
}

function fShowForms() {
  // dynamically process a list of forms to hide them
  for (var vIndex = 0; vIndex < arguments.length; vIndex++) {
    document.all[arguments[vIndex].name].style.display = 'block';
  }
}

function fHideShowObject() {
  // dynamically process a list of HTML objects (e.g., tables) to toggle their display state
  with (document) {
    var vObject;
    var vSetting;
    for (var vIndex = 0; vIndex < arguments.length; vIndex++) {
      vObject   = arguments[vIndex];
      vSetting  = (vObject.style.display == 'block') ? 1 : 0;
      vSetting  = fCycleStyle(vSetting, vObject.id, 2)
      vObject.style.display = (vSetting == '0') ? 'none' : 'block';
    }
  }
}

function fLocationButtonTest() {
  // dynamically process a list of forms to hide them;
  // if any of the forms AFTER the first 2 in the list are displayed,
  // then show the first form, else hide the first form, and
  // show the 2nd form only if the 1st form is shown.
  var vTally = 'none';
  for (var vIndex = 2; vIndex < arguments.length; vIndex++) {
    if (document.all[arguments[vIndex].name].style.display == 'block') {
      vTally = 'block';
    }
  }
  document.all[arguments[0].name].style.display = vTally;
  if (document.all[arguments[0].name].style.display == 'none') {
    document.all[arguments[1].name].style.display = 'none';
  }
}

function fShowSelectedForms() {
  // loop through ALL forms and hide them
  var vNumberForms = document.forms.length;
  for (var vIndex = 0; vIndex < vNumberForms; vIndex++) {
    document.forms[vIndex].style.display = 'none';
  }
  // loop through ALL arguments and show them
  for (var vIndex = 0; vIndex < arguments.length; vIndex++) {
    document.forms[arguments[vIndex].name].style.display = 'block';
  }
}

function fHideShowRows(argType) {
  // load and initialize setting to hide or display various objects
  with (document) {
    switch (argType) {
      case 'Override' :
        if (frmLocation.frmLocationTimeOverride.checked == false) { // if not checked, toggle display, otherwise force display on
          frmLabelTimeOverride.style.display = frmLabelTimeOverride.style.display == 'block' ? 'none' : 'block';
          break;
        }
    }
  }
}

function fRefresh(argForm) {
  // reload plugin if closing Settings window
  if (argForm.style.display == 'none') { window.location.reload(false); }
}

function fBuildSelect(argFormField, argStart, argStop, argDelta, argSelected, argPrefix, argSuffix) {
  // Builds numeric Option values for HTML Select statement
  // Arguments:
  //  argFormField : form Select field to build numeric options for
  //  argStart     : start integer; may be negative or positive
  //  argStart     : stop integer; may be negative or positive
  //  argDelta     : increment integer; treated as a positive number
  //  argSelected  : initial selected value
  //  argPrefix    : string to precede number value
  //  argSuffix    : string to follow number value (e.g., '%')
  var vValueStr;
  var vSpan   = Math.round(Math.abs((argStart - argStop)/argDelta));
  var vDelta  = Math.abs(argDelta);
  var vValue  = argStart;
  var vStrLen = (String(argStart).length > String(argStop).length) ? String(argStart).length : String(argStop).length;

  for (var vIndex = 0; vIndex <= vSpan; vIndex++) {
    vValueStr = String(vValue);
    while (vValueStr.length < vStrLen) {vValueStr = ' ' + vValueStr;}

    argFormField[vIndex]  = new Option (text=argPrefix+vValueStr+argSuffix, value=vValue);
    vValue                = ((argStart < argStop) ? vValue + vDelta : vValue - vDelta);
  }
  argFormField.value = (Math.abs(argSelected) >= Math.abs(argStart) && Math.abs(argSelected) <= Math.abs(argStop)) ? argSelected : argStart;
}

function fShowHelp(){
  var vHelpType = fReadINI(kSettingsFile, 'General', 'HelpType', 'Tab');
  switch (vHelpType) {

    case 'Tab':
      var vFolder = external.m2_plugin_folder(max_security_id,kPluginName);
      window.open(vFolder + '\Help.htm');
      break;
      
    case 'Popup':
      external.max_modelessDialog(max_security_id,'Help.htm',window,'status:no;help:no;resizable:yes;scroll:yes;dialogWidth:800px;dialogHeight:650px',window)
      break;
    }
}

function fSaveHelpType(argRadioBtn){
  fWriteINI(kSettingsFile, 'General', 'HelpType', argRadioBtn);
}

function max_callback(argParameter){
  switch (argParameter) {
//     case 'sidebar_activate'   : alert (argParameter); break;
//     case 'sidebar_deactivate' : alert (argParameter); break;
//     case 'sidebar_tab_change' : alert (argParameter); break;
//     case 'sidebar_unload'     : alert (argParameter); break;
  }
}


//-->
