/*
History
=======
Ver. 1.0  2003-11-14  Initial release by John Land
Ver. 1.1  2003-11-15  Added default content function for HTML text, so if
                      language file is missing, all HTML content is still
                      rendered in English
Ver. 1.2  2003-11-19  Tidied up code, simplifying some functions;
                      Added 4 overlooked script text message variables;
                      Added several new tool tips, changed several others;
Ver. 1.3  2004-04-30  Added "frmUseLocalTime.value"
Ver. 1.4  2004-07-08  Made MaxThon compatible
Ver. 1.5  2004-12-11  Added vglobalMondayStart variable
Ver. 1.6  2005-10-27  Major revamp to reflect changes to main program
                      and streamline configuration code
Ver. 1.7  2005-12-24  Minor changes to default language selection;
                      a default language other than English can be defined by
                      editing the DefaultLanguage parameter in Settings.ini
*/

//=============================
// GLOBAL CONSTANTS & VARIABLES
//=============================

var kPluginName         = 'WeekCalendar';  // name of this plugin
var kSettingsFile       = 'Settings.ini';  // name of associated settings file
var kCountriesFile      = 'Countries.ini'; // name of countries file
var kSpecialDatesFile   = 'Dates.ini'      // name of holiday and diary file
var kLocationsFile      = 'Locations';     // ROOT name of geographic locations files
var kSoundFiles         = 'sounds/';       // subdirectory for alarm sounds
var kImageFiles         = 'images/';       // subdirectory for icons & Moon phase images

var kSelectedDayColor   = 'black';
var kSelectedDayBgColor = 'yellow';
var kNormalDayColor     = 'blue';
var kNormalDayBgColor   = 'snow';
var kWeekEndColor       = 'gainsboro';
var kTodayColor         = 'black';
var kTodayBgColor       = 'aqua';
var kHolidayColor       = 'red';
var kHolidayBGColor     = 'greenyellow';
var kUserDateBGColor    = '#ffc8c8';

var vglobalMondayStart  = 0;  // Global for starting the calendar with Sunday (=false) or Monday (=true)
var vglobalDateStyle    = 1;  // Global for date style -- true=YYYYMMDD, false=DDMMYYYY
var vglobalMonthStyle   = 1;  // Global for month style -- true=short name, false=long name
var vglobalClockStyle   = 1;  // Global for clock style --          0=am/pm, 1=24hr, 2=Beat time, 3=utc 24 hour
var vglobalLocTimeStyle = 0;  // Global for location time style --  0=am/pm, 1=24hr, 2=utc 24 hour
var vglobalLocaleStyle  = 1;  // Global for locale time style --    0=am/pm, 1=24hr, 2=utc 24 hour
var vglobalTZStyle      = 0;  // Global for time zone time style -- 0=am/pm, 1=24hr
var vglobalEasterStyle  = [0,1];  // Global for Easter event; 2nd number is number of events
var vglobalAlarmLoop    = true;   // Global for momentarily enabling/disabling Alarm check routine

// MUST first read number of date calculator, time zone, & alarm rows to build
var vglobalNumDateArithmeticRows = +(fReadINI(kSettingsFile, 'AppStyle', 'NumberDateArithmeticRows', '1'));
var vglobalNumDateDiffRowPairs   = +(fReadINI(kSettingsFile, 'AppStyle', 'NumberDateDiffRowPairs',   '1'));
var vglobalNumCountdownRows      = +(fReadINI(kSettingsFile, 'AppStyle', 'NumberCountDownRows',      '3'));
var vglobalNumberTimeZoneRows    = +(fReadINI(kSettingsFile, 'AppStyle', 'NumberTimeZoneRows',       '3'));
var vglobalNumberAlarmRows       = +(fReadINI(kSettingsFile, 'AppStyle', 'NumberAlarmRows',          '1'));

// set default values for ALL global variables holding text used in alerts & constructed strings
var msgNaN                ='Input is not a number';
var msgEaster             ='Easter Sunday';
var msgAdvent             ='Advent';
var msgOutOfCalendarRange ='was the first full year of the Gregorian Calendar.\nThis calendar does not account for prior dates.';
var msgOutOfSkyTimesRange ='The algorithm for sun times used is \nnot valid for years outside of \nthe range -1000 to 3000.';
var msgLatTooCloseToPoles ='All latitudes below 8948\'0\' S or above 8948\'0\' N\n will be reset to 8948\'0\' (89.8) N or S, respectively';
var msgNoSunriseFound     ='Cannot Find Sunrise!';
var msgNoSunsetFound      ='Cannot Find Sunset!';
var msgNotApplicable      ='N/A';
var msgHours              ='hours';
var msgMinutes            ='minutes';
var msgDegrees            ='';
var msg_H                 ='h';
var msg_M                 ='m';
var msg_Min               ='min';
var msg_Day               ='Day';
var msg_Week              ='Week';
var msg_of                ='of';
var msg_left              ='left';
var msg_am                ='am';
var msg_pm                ='pm';
var msg_utc               ='utc';
var msgSetLocationClosed  ='Set Location';
var msgSetLocationOpened  ='Close Location Settings';
var msgSkyInfo            ='Values as of';
var msgNoCitySelection    ='No city selected!';
var msgNoSnoozeSet        ='No snooze time set';
var msgNoAlarmSettings    ='No settings for this alarm';
var msgNoMessage          ='[No message]';
var msgAlarmSound         ='Alarm Sound: ';
var msgMessage            ='Message: ';
var msgCounterRepeats     ='Counter repeats every ';
var msgAlarmSnoozes       ='Alarm snoozes for ';
var msgNoRise             ='No rise';
var msgNoSet              ='No set';
var msgMoonDown           ='Moon down';
var msgMoonUp             ='Moon up';
var msgDiaryEvent         ='<enter event for selected date>';

// set default values for day name and month name arrays
var kLongMonthName  =['January','February','March','April','May','June','July','August','September','October','November','December'];
var kShortMonthName =['Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec'];
var kLongDayName    =['Monday','Tuesday','Wednesday','Thursday','Friday','Saturday','Sunday'];
var kShortDayName   =['Mo','Tu','We','Th','Fr','Sa','Su'];
var kMonthLength    =[31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];

//=====================
// Read/Write functions
//=====================

function fReadINIArray(argINIFile, argSection, argKey, argDefault) {
  // routine for reading an returning an array of values from an INI file
  // last element of external.m2_readIni is the default value if no INI value exists
  var vItem = (external.m2_readIni(max_security_id, kPluginName, argINIFile, argSection, argKey, argDefault));
  if (vItem == "??") {
     return (argDefault);
  } else {
    return (vItem.split(','));
  }
}

function fWriteINI(argINIFile, argSection, argKey, argWriteValue) {
  external.m2_writeIni(max_security_id, kPluginName, argINIFile, argSection, argKey, argWriteValue.toString());
}

function fReadINI(argINIFile, argSection, argKey, argDefault) {
  // last element of external.m2_readIni is the default value if no INI value exists
  return (external.m2_readIni(max_security_id, kPluginName, argINIFile, argSection, argKey, argDefault));
}

//==========================================
// Initialization & language setup functions
//==========================================

function fReadLanguageFile() {
  var vLangFile         = fGetLanguage();
  
  // set variable either to language file content if it exists,
  // OR to the existing default value for the variable, set above.
  kLongMonthName        = fReadINIArray(vLangFile,'Script Text','kLongMonthName',  kLongMonthName);
  kShortMonthName       = fReadINIArray(vLangFile,'Script Text','kShortMonthName', kShortMonthName);
  kLongDayName          = fReadINIArray(vLangFile,'Script Text','kLongDayName',    kLongDayName);
  kShortDayName         = fReadINIArray(vLangFile,'Script Text','kShortDayName',   kShortDayName);

  msgOutOfCalendarRange = fReadINI(vLangFile,'Script Text','msgOutOfCalendarRange', msgOutOfCalendarRange);
  msgNaN                = fReadINI(vLangFile,'Script Text','msgNaN', msgNaN);
  msgEaster             = fReadINI(vLangFile,'Script Text','msgEaster', msgEaster);
  msgAdvent             = fReadINI(vLangFile,'Script Text','msgAdvent', msgAdvent);
  msgOutOfSkyTimesRange = fReadINI(vLangFile,'Script Text','msgOutOfSkyTimesRange',msgOutOfSkyTimesRange);
  msgLatTooCloseToPoles = fReadINI(vLangFile,'Script Text','msgLatTooCloseToPoles',msgLatTooCloseToPoles);
  msgNoSunriseFound     = fReadINI(vLangFile,'Script Text','msgNoSunriseFound', msgNoSunriseFound);
  msgNoSunsetFound      = fReadINI(vLangFile,'Script Text','msgNoSunsetFound', msgNoSunsetFound);
  msgNotApplicable      = fReadINI(vLangFile,'Script Text','msgNotApplicable', msgNotApplicable);
  msgMinutes            = fReadINI(vLangFile,'Script Text','msgMinutes', msgMinutes);
  msgDegrees            = fReadINI(vLangFile,'Script Text','msgDegrees', msgDegrees);
  msg_H                 = fReadINI(vLangFile,'Script Text','msg_H', msg_H);
  msg_M                 = fReadINI(vLangFile,'Script Text','msg_M', msg_M);
  msg_Min               = fReadINI(vLangFile,'Script Text','msg_Min', msg_Min);
  msg_Day               = fReadINI(vLangFile,'Script Text','msg_Day', msg_Day);
  msg_Week              = fReadINI(vLangFile,'Script Text','msg_Week', msg_Week);
  msg_of                = fReadINI(vLangFile,'Script Text','msg_of', msg_of);
  msg_left              = fReadINI(vLangFile,'Script Text','msg_left', msg_left);
  msg_am                = fReadINI(vLangFile,'Script Text','msg_am', msg_am);
  msg_pm                = fReadINI(vLangFile,'Script Text','msg_pm', msg_pm);
  msg_utc               = fReadINI(vLangFile,'Script Text','msg_utc', msg_utc);
  msgSetLocationClosed  = fReadINI(vLangFile,'Script Text','msgSetLocationClosed', msgSetLocationClosed);
  msgSetLocationOpened  = fReadINI(vLangFile,'Script Text','msgSetLocationOpened', msgSetLocationOpened);
  msgSkyInfo            = fReadINI(vLangFile,'Script Text','msgSkyInfo', msgSkyInfo);
  msgNoCitySelection    = fReadINI(vLangFile,'Script Text','msgNoCitySelection', msgNoCitySelection);
  msgNoSnoozeSet        = fReadINI(vLangFile,'Script Text','msgNoSnoozeSet', msgNoSnoozeSet);
  msgNoAlarmSettings    = fReadINI(vLangFile,'Script Text','msgNoAlarmSettings', msgNoAlarmSettings);
  msgNoMessage          = fReadINI(vLangFile,'Script Text','msgNoMessage', msgNoMessage);
  msgAlarmSound         = fReadINI(vLangFile,'Script Text','msgAlarmSound', msgAlarmSound);
  msgMessage            = fReadINI(vLangFile,'Script Text','msgMessage', msgMessage);
  msgCounterRepeats     = fReadINI(vLangFile,'Script Text','msgCounterRepeats', msgCounterRepeats);
  msgAlarmSnoozes       = fReadINI(vLangFile,'Script Text','msgAlarmSnoozes', msgAlarmSnoozes);
  msgNoRise             = fReadINI(vLangFile,'Script Text','msgNoRise', msgNoRise);
  msgNoSet              = fReadINI(vLangFile,'Script Text','msgNoSet', msgNoSet);
  msgMoonDown           = fReadINI(vLangFile,'Script Text','msgMoonDown', msgMoonDown);
  msgMoonUp             = fReadINI(vLangFile,'Script Text','msgMoonUp', msgMoonUp);
  msgDiaryEvent         = fReadINI(vLangFile,'Script Text','msgDiaryEvent', msgDiaryEvent);

  // set HTML element either to language file content if it exists,
  // or to the existing default content for the element, set in the HTML.
  fSetHTMLTextLoop(vLangFile,'HTML Repetitive Text','LB.title',                            0, 6);
  fSetHTMLTextLoop(vLangFile,'HTML Repetitive Text','frmAlarmOn.title',                    1, vglobalNumberAlarmRows);
  fSetHTMLTextLoop(vLangFile,'HTML Repetitive Text','frmAlarmSet.title',                   1, vglobalNumberAlarmRows);
  fSetHTMLTextLoop(vLangFile,'HTML Repetitive Text','frmAlarmSnoozeTypeDisplay.title',     1, vglobalNumberAlarmRows);
  fSetHTMLTextLoop(vLangFile,'HTML Repetitive Text','frmAlarmTime.title',                  1, vglobalNumberAlarmRows);
  fSetHTMLTextLoop(vLangFile,'HTML Repetitive Text','frmAlarmTypeDisplay.title',           1, vglobalNumberAlarmRows);
  fSetHTMLTextLoop(vLangFile,'HTML Repetitive Text','frmDateCalcBetweenDatesFirst.title',  1, vglobalNumDateDiffRowPairs);
  fSetHTMLTextLoop(vLangFile,'HTML Repetitive Text','frmDateCalcBetweenDatesSecond.title', 1, vglobalNumDateDiffRowPairs);
  fSetHTMLTextLoop(vLangFile,'HTML Repetitive Text','frmDateCalcCountDownChkBox.title',    1, vglobalNumCountdownRows);
  fSetHTMLTextLoop(vLangFile,'HTML Repetitive Text','frmDateCalcDateSelectFirst.title',    1, vglobalNumDateDiffRowPairs);
  fSetHTMLTextLoop(vLangFile,'HTML Repetitive Text','frmDateCalcDateSelectSecond.title',   1, vglobalNumDateDiffRowPairs);
  fSetHTMLTextLoop(vLangFile,'HTML Repetitive Text','frmDateCalcDateToCountFrom.title',    1, vglobalNumCountdownRows);
  fSetHTMLTextLoop(vLangFile,'HTML Repetitive Text','frmDateCalcDayDelta.title',           1, vglobalNumDateArithmeticRows);
  fSetHTMLTextLoop(vLangFile,'HTML Repetitive Text','frmDateCalcDaysBetweenDates.title',   1, vglobalNumDateDiffRowPairs);
  fSetHTMLTextLoop(vLangFile,'HTML Repetitive Text','frmDateCalcDaysFromDate.title',       1, vglobalNumDateArithmeticRows);
  fSetHTMLTextLoop(vLangFile,'HTML Repetitive Text','frmDateCalcDaysToGo.title',           1, vglobalNumCountdownRows);
  fSetHTMLTextLoop(vLangFile,'HTML Repetitive Text','frmDateCalcDiff.innerHTML',           1, vglobalNumDateDiffRowPairs);
  fSetHTMLTextLoop(vLangFile,'HTML Repetitive Text','frmDateCalcOperation.title',          1, vglobalNumDateArithmeticRows);
  fSetHTMLTextLoop(vLangFile,'HTML Repetitive Text','frmDateCalcToGo.innerHTML',           1, vglobalNumCountdownRows);
  fSetHTMLTextLoop(vLangFile,'HTML Repetitive Text','frmTimeZone.title',                   1, vglobalNumberTimeZoneRows);
  fSetHTMLTextLoop(vLangFile,'HTML Repetitive Text','frmTimeZoneDST.title',                1, vglobalNumberTimeZoneRows);
  fSetHTMLTextLoop(vLangFile,'HTML Repetitive Text','frmTimeZoneLocation.title',           1, vglobalNumberTimeZoneRows);
  fSetHTMLTextLoop(vLangFile,'HTML Repetitive Text','frmTimeZoneTime.title',               1, vglobalNumberTimeZoneRows);

  // this loop reads a sequence of HTML text from the designated language file;
  // the items must be in continuous numeric order
  for (var vIndex = 1; vIndex; vIndex++) { // Loop until no more items to read
    var vHTML_Array = fReadINI(vLangFile, 'HTML Text', String(vIndex), 'None').split('=');
    if (vHTML_Array != 'None') {
      fSetHTMLText(vHTML_Array[0], vHTML_Array[1]);
    } else {
      break;
    }
  }
}

function fGetLanguage() {
  switch (external.max_language_id) {
    case '0000': vLangFile = 'en.ini'; break;
    case '0001': vLangFile = 'zh.ini'; break;
    case '0002': vLangFile = 'ar.ini'; break;
    case '0003': vLangFile = 'bg.ini'; break;
    case '0004': vLangFile = 'ca.ini'; break;
    case '0006': vLangFile = 'hr.ini'; break;
    case '0007': vLangFile = 'cs.ini'; break;
    case '0008': vLangFile = 'da.ini'; break;
    case '0009': vLangFile = 'nl.ini'; break;
    case '0043': vLangFile = 'fa.ini'; break;
    case '0041': vLangFile = 'fi.ini'; break;
    case '0010': vLangFile = 'fr.ini'; break;
    case '0044': vLangFile = 'ry.ini'; break;
    case '0013': vLangFile = 'de.ini'; break;
    case '0015': vLangFile = 'el.ini'; break;
    case '0016': vLangFile = 'he.ini'; break;
    case '0017': vLangFile = 'hu.ini'; break;
    case '0018': vLangFile = 'it.ini'; break;
    case '0019': vLangFile = 'ja.ini'; break;
    case '0020': vLangFile = 'ko.ini'; break;
    case '0021': vLangFile = 'lv.ini'; break;
    case '0023': vLangFile = 'no.ini'; break;
    case '0024': vLangFile = 'pl.ini'; break;
    case '0027': vLangFile = 'pt.ini'; break;
    case '0029': vLangFile = 'sr.ini'; break;
    case '0040': vLangFile = 'ro.ini'; break;
    case '0028': vLangFile = 'ru.ini'; break;
    case '0030': vLangFile = 'sk.ini'; break;
    case '0042': vLangFile = 'sl.ini'; break;
    case '0031': vLangFile = 'es.ini'; break;
    case '0032': vLangFile = 'sv.ini'; break;
    case '0033': vLangFile = 'tt.ini'; break;
    case '0034': vLangFile = 'th.ini'; break;
    case '0035': vLangFile = 'tr.ini'; break;
    case '0036': vLangFile = 'uk.ini'; break;
    default    : vLangFile = 'en.ini';
  }

  // try to read something from vLangFile; if not read, assume file is missing
  // and select the English file en.ini
  var vTest         = fReadINI(vLangFile,'Script Text','msgNaN', 'error');
  var vDefaultLang  = fReadINI(kSettingsFile,'General','DefaultLanguage', 'en.ini');
  vLangFile = (vTest == 'error') ? vDefaultLang : (vDefaultLang != vLangFile ? vDefaultLang : vLangFile);
  return (vLangFile);
}

function fSetHTMLText(argElement, argText) {
  // figure out HTML element type and set element to language file content
  var vArgArray = argElement.split('.');
  try {
    eval ('document.getElementById("'+vArgArray[0]+'").'+vArgArray[1]+' = argText');
  }
  catch (err){}
}

function fSetHTMLTextLoop(argLangFile, argSection, argKey, argStart, argStop) {
  // for items that use the same text, loop through a generic key item,
  // and add an index to distinguish specific items.
  var vArgArray = argKey.split('.');
  for (var vIndex = +argStart; vIndex <= +argStop; vIndex++) {
    var vElement = vArgArray[0]+vIndex;
    try {
      eval ('document.getElementById("'+vElement+'").'+vArgArray[1]+' = fReadINI(argLangFile, argSection, argKey, document.getElementById("'+vElement+'").'+vArgArray[1]+')');
    }
    catch (err){}
  }
}

function fSaveLocSettings(argForm) {
  // Write location coordinates and time zone information
  with (argForm) {
    var vLatitudeValues  = [frmLatDegInput.value,frmLatMinInput.value,frmLatSecInput.value,frmLocationLatDirInput.value];
    fWriteINI(kSettingsFile, 'Location', 'Latitude', vLatitudeValues);
    
    var vLongitudeValues = [frmLongDegInput.value,frmLongMinInput.value,frmLongSecInput.value,frmLocationLongDirInput.value];
    fWriteINI(kSettingsFile, 'Location', 'Longitude', vLongitudeValues);

    var vTimeZoneValues  = [frmLocationUTCOffsetInput.value,frmLocationUTCAdjustInput.value,frmLocationDSTInput.value];
    fWriteINI(kSettingsFile, 'Location', 'TimeZone', vTimeZoneValues);
  }
}

function fReadLocSettings(argForm) {
  // Read location coordinates and time zone information
   with (argForm) {
     var vLatitudeValues   = fReadINI(kSettingsFile, 'Location', 'Latitude' , [0,0,0,0]).split(',');
     frmLatDegInput.value            = vLatitudeValues[0];
     frmLatMinInput.value            = vLatitudeValues[1];
     frmLatSecInput.value            = vLatitudeValues[2];
     frmLocationLatDirInput.value    = vLatitudeValues[3];

     var vLongitudeValues  = fReadINI(kSettingsFile, 'Location', 'Longitude' , [0,0,0,0]).split(',');
     frmLongDegInput.value           = vLongitudeValues[0];
     frmLongMinInput.value           = vLongitudeValues[1];
     frmLongSecInput.value           = vLongitudeValues[2];
     frmLocationLongDirInput.value   = vLongitudeValues[3];
     
     var vTimeZoneValues   = fReadINI(kSettingsFile, 'Location', 'TimeZone' , ['0','0.00','0']).split(',');
     frmLocationUTCOffsetInput.value = vTimeZoneValues[0];
     frmLocationUTCAdjustInput.value = vTimeZoneValues[1];
     frmLocationDSTInput.value       = vTimeZoneValues[2];
   }
}

function fSaveNumRows(argForm){
  // save Settings table info
  with (argForm){
    fWriteINI(kSettingsFile, 'AppStyle', 'NumberDateArithmeticRows', frmNumDateArithmeticRows.value);
    fWriteINI(kSettingsFile, 'AppStyle', 'NumberDateDiffRowPairs',   frmNumDateDiffRowPairs.value);
    fWriteINI(kSettingsFile, 'AppStyle', 'NumberCountDownRows',      frmNumDateCountdownRows.value);
    fWriteINI(kSettingsFile, 'AppStyle', 'NumberTimeZoneRows',       frmNumTimeZoneRows.value);
    fWriteINI(kSettingsFile, 'AppStyle', 'NumberAlarmRows',          frmNumAlarmRows.value);
  }
}

function fReSetBackgroundColor() {
  var vColor = fReadINI(kSettingsFile, 'AppStyle', 'BackgroundColor', 'darkturquoise');
	vColor = dlgHelper.ChooseColorDlg(vColor);
	vColor = vColor.toString(16);
  if (vColor.length < 6) {
    var vTempString = "000000".substring(0,6-vColor.length);
    vColor = vTempString.concat(vColor);
  }
  vColor = "#" + vColor;
  document.body.bgColor = vColor;
  fWriteINI(kSettingsFile, 'AppStyle', 'BackgroundColor', vColor);
}

function fSetBackgroundColor() {
  document.body.bgColor = fReadINI(kSettingsFile, 'AppStyle', 'BackgroundColor', 'darkturquoise');
}

function fInitDateCalculator(argForm) {
  // initialize the date arithmetic calculator with saved operations and values;
  // initialize the 'Date Difference' and 'Days to Go' calculators with saved Julian date values.
  //
  // The "eval" function creates JS commands out of strings, allowing a loop
  // through the similarly named form fields; shorter and more general than old code.
  // vglobalNumDateArithmeticRows is a global variable set in the main program file
  // vglobalNumDateDiffRowPairs is a global variable set in the main program file
  // vglobalNumCountdownRows is a global variable set in the main program file
  with (argForm) {
     for (var i = 1; i <= vglobalNumDateArithmeticRows; i++) {
      eval("frmDateCalcOperation"+i+".defaultSelected = fReadINI(kSettingsFile, 'DateCalc', frmDateCalcOperation"+i+".id, '+')");
      eval("frmDateCalcOperation"+i+".value = frmDateCalcOperation"+i+".defaultSelected;");
      eval("frmDateCalcDayDelta"+i+".value  = fReadINI(kSettingsFile, 'DateCalc', frmDateCalcDayDelta"+i+".id, '')");
    }
    for (var i=1; i<=vglobalNumDateDiffRowPairs; i++) {
      eval("frmJDateFirst"+i+".value  = fReadINI(kSettingsFile, 'DateCalc', frmJDateFirst"+i+".id, 0)");
      eval("frmJDateSecond"+i+".value = fReadINI(kSettingsFile, 'DateCalc', frmJDateSecond"+i+".id, 0)");
    }
    for (var i=1; i<=vglobalNumCountdownRows; i++) {
      eval("frmJDateCntDn"+i+".value  = fReadINI(kSettingsFile, 'DateCalc', frmJDateCntDn"+i+".id, 0)");
    }
  }
}

function fInitTimeZones(argForm){
  // loop through all Time Zone fields, reading the stored value and setting each field appropriately
  with (argForm) {
    for (var i = 1; i <= vglobalNumberTimeZoneRows; i++) {
      eval("var vLocationArray = fReadINI(kSettingsFile, 'TimeZoneLocations', 'frmTimeZoneLocation"+i+"', 'None').split(',')");
      if (vLocationArray != 'None' && vLocationArray != ''){// skip over empty rows
        eval("frmTimeZone"+i+".checked       = 1");
        var vOffset = (Math.abs(vLocationArray[11]) + (+vLocationArray[12]));
        vOffset = (vLocationArray[11] < 0 ? -vOffset : vOffset); // preserve sign
        eval("frmTimeZoneOffset"+i+".value   = vOffset");
        eval("var vDST = fReadINI(kSettingsFile, 'TimeZoneLocations', 'frmTimeZoneDST"+i+"', 'false')");
        eval("frmTimeZoneDST"+i+".checked    = (vDST == 'false' ? 0 : 1)");
        eval("frmTimeZoneLocation"+i+".value = vLocationArray[2]");
      } else {  // set empty rows to zero or empty strings
        eval("frmTimeZone"+i+".checked       = 0");
        eval("frmTimeZoneLocation"+i+".value = ''");
        eval("frmTimeZoneOffset"+i+".value   = ''");
        eval("frmTimeZoneDST"+i+".checked    = 0");
      }
    }
    frmTimeZoneSelectFlag.value = '';
  }
}

function fInitAlarms(argForm){
  // loop through all Alarm fields, reading the stored value and setting each field appropriately
  with (argForm) {
    for (var i = 1; i <= vglobalNumberAlarmRows; i++) {
      eval("var vSettingsArray = fReadINI(kSettingsFile, 'Alarms', frmAlarmTime"+i+".id, '').split(',');");
      if (vSettingsArray != '') {
        eval("frmAlarmSet"+i+".checked      = true;");
        eval("frmAlarmOn"+i+".checked       = vSettingsArray[0] == 'true' ? true : false;");
        eval("frmCounterFlag"+i+".value     = +vSettingsArray[1];");

        if (+vSettingsArray[1] == 0) { // don't convert format for counters
          eval("frmAlarmTime"+i+".value     = fHMFormat(vSettingsArray[3], vSettingsArray[4]);");
        } else {
          eval("frmAlarmTime"+i+".value     = fHMTimeString(vSettingsArray[3], vSettingsArray[4], true);");
        }
        eval("frmAlarmTimeSet"+i+".value    = vSettingsArray[3]*60.0 + (+vSettingsArray[4]);");
        eval("frmAlarmSnoozeTime"+i+".value = vSettingsArray[5]*60.0 + (+vSettingsArray[6]);"); // in minutes

        var vSnoozeTime                     = eval("frmAlarmSnoozeTime"+i+".value");
        if (vSnoozeTime == 0 && +vSettingsArray[1] == 1) { // set default snooze time for counters
          eval("frmAlarmSnoozeTime"+i+".value = vSettingsArray[3]*60.0 + (+vSettingsArray[4]);"); // in minutes
        }
        var vSnoozeSet                      = eval("frmAlarmSnoozeTime"+i+".value");
        var vSnoozeFlag                     = (vSnoozeSet == 0) ? 0 : 1;  // set flag if snooze delay not equal to zero

        var vNow = new Date();
        eval("frmResidualSeconds"+i+".value = vNow.getSeconds();");
        eval("frmAlarmRang"+i+".value       = 0;");
        eval("frmAlarmTypeDisplay"+i+".src  = kImageFiles + (vSettingsArray[1] == '1' ? 'HourGlass.gif' : 'AlarmClock.gif');");
        eval("frmAlarmSnoozeTypeDisplay"+i+".src = kImageFiles +  (vSnoozeFlag == 1   ? 'Recycle.gif'   : 'Blank15x16.gif');");
      }
    }
    frmAlarmSound.value = fReadINI(kSettingsFile, 'Alarms', 'LastSelectedSound', 'none');  // read default sound
  }
}

function fLoadSetup(argForm) {
  var vHelpType = fReadINI(kSettingsFile, 'General', 'HelpType', 'Tab');
  switch (vHelpType) {
    case 'Tab':
      argForm.frmLabelSettingsHelpTypeTabRB.checked = true;
      break;
    case 'Popup':
      argForm.frmLabelSettingsHelpTypePopupRB.checked = true;
      break;
  }
}

function fLoadAppStyles() {
  // Read global style information
  vglobalMondayStart    = +fReadINI(kSettingsFile, 'AppStyle', 'MondayStart', '0');
  vglobalDateStyle      = +fReadINI(kSettingsFile, 'AppStyle', 'DateStyle',   '1');
  vglobalMonthStyle     = +fReadINI(kSettingsFile, 'AppStyle', 'MonthStyle',  '1');
  vglobalClockStyle     = +fReadINI(kSettingsFile, 'AppStyle', 'ClockStyle',  '1');
  vglobalLocTimeStyle   = +fReadINI(kSettingsFile, 'AppStyle', 'LocTimeStyle','0');
  vglobalLocaleStyle    = +fReadINI(kSettingsFile, 'AppStyle', 'LocaleStyle', '1');
  vglobalTZStyle        = +fReadINI(kSettingsFile, 'AppStyle', 'TZStyle',     '0');
  vglobalEasterStyle[0] = +fReadINI(kSettingsFile, 'AppStyle', 'EasterStyle', '0');
}

function fSetDisplayState(argForm) {
  // read 0 or 1 setting & set display value to 'none' or 'block', respectively
  var vSetting = fReadINI(kSettingsFile, 'AppStyle', argForm.id, 'none');
  switch (vSetting){
    case '0' : argForm.style.display = 'none';  break;
    case '1' : argForm.style.display = 'block'; break;
  }
}

function fHide(argForm){
  with (document){
    argForm.style.display='none';
  }
}

function fShow(argForm){
  with (document){
    argForm.style.display='block';
  }
}

function fLoadHideShowSettings() {
  // load and initialize setting to hide or display various forms
  with (document) {
    fSetDisplayState(frmCalendar);// set to stored state, but see below
    fSetDisplayState(frmCalendarBody);
    fSetDisplayState(frmCalendarData);
    fSetDisplayState(frmDateCalc);
    fSetDisplayState(frmSunTimes);
    fSetDisplayState(frmMoonTimes);
    fSetDisplayState(frmTimeZones);
    fSetDisplayState(frmTableDateArithmetic);
    fSetDisplayState(frmTableDateDifference);
    fSetDisplayState(frmTableDateCounter);
    fSetDisplayState(frmTableTwilightAM);
    fSetDisplayState(frmTableTwilightPM);
    fSetDisplayState(frmTableSunExtra);
    fSetDisplayState(frmTableMoonExtra);
    fSetDisplayState(frmAlarm);

    fHide(frmTableTimeZones);
    fHide(frmLocation);
    fHide(frmTimeZoneError);
    fHide(frmTableAlarmSetup);
    fHide(frmLabelTimeOverride);
    fHide(frmSetup);

    frmLocation.frmLocationTimeOverride.checked = false;  //reset Time Override button
    frmLocationButton.frmLocationHideShow.value = msgSetLocationClosed;

    // force display of the calendar if any of certain other tables are displayed
    if (frmCalendarData.style.display == 'block' || frmDateCalc.style.display  == 'block' ||
        frmSunTimes.style.display     == 'block' || frmMoonTimes.style.display == 'block' ||
        frmTimeZones.style.display    == 'block') {
      fShow(frmCalendar);
    }

    // display the Set Location button if any of certain other tables are displayed
    fHide(frmLocationButton); // default is 'not displayed'
    if (frmSunTimes.style.display  == 'block' || frmMoonTimes.style.display == 'block' ||
        frmTimeZones.style.display == 'block') {
      fShow(frmLocationButton);
    }
  }
}

function fMasterConfiguration(){
  // set language words/phrases for user interface; do first to avoid script failures
  fReadLanguageFile();
  
  fSetBackgroundColor();

  // build select buttons, specifying start, stop, and default selected nos.
  with (document.frmLocation) {
    fBuildSelect(frmLatDegInput   ,0,90,1,45,'','');
    fBuildSelect(frmLatMinInput   ,0,59,1,0,'','');
    fBuildSelect(frmLatSecInput   ,0,59,1,0,'','');
    fBuildSelect(frmLongDegInput  ,0,180,1,90,'','');
    fBuildSelect(frmLongMinInput  ,0,59,1,0,'','');
    fBuildSelect(frmLongSecInput  ,0,59,1,0,'','');
    fBuildSelect(frmLocationUTCOffsetInput,14,-12,1,0,'','');
    fBuildSelect(frmLocationHoursInput    ,0,23,1,0,'','');
    fBuildSelect(frmLocationMinutesInput  ,0,59,1,0,'','');
    fBuildSelect(frmLocationSecondsInput  ,0,59,1,0,'','');
  }
  with (document.frmSetup) {
    fBuildSelect(frmNumDateArithmeticRows,1,9,1,  vglobalNumDateArithmeticRows,'','');
    fBuildSelect(frmNumDateDiffRowPairs  ,1,9,1,  vglobalNumDateDiffRowPairs,'','');
    fBuildSelect(frmNumDateCountdownRows ,1,9,1,  vglobalNumCountdownRows,'','');
    fBuildSelect(frmNumTimeZoneRows      ,1,24,1, vglobalNumberTimeZoneRows,'','');
    fBuildSelect(frmNumAlarmRows         ,1,9,1,  vglobalNumberAlarmRows,'','');
  }
  with (document.frmAlarm) {
    fBuildSelect(frmAlarmHoursInput         ,0,23,1,0,'','');
    fBuildSelect(frmAlarmMinutesInput       ,0,59,1,0,'','');
    fBuildSelect(frmAlarmSnoozeHoursInput   ,0,23,1,0,'','');
    fBuildSelect(frmAlarmSnoozeMinutesInput ,0,59,1,0,'','');
  }

  // load saved application style settings
  fLoadAppStyles();

  // load date calculator settings
  fInitDateCalculator(frmDateCalc);

  // load location (lat., long., time zone) settings
  fReadLocSettings(frmLocation);

  // load time zone settings
  fInitTimeZones(frmTimeZones)

  // load time zone countries
  fLoadTZCountries(frmTimeZones);

  // load alarm sounds
  fLoadAlarmSounds(frmAlarm);

  // load alarm settings
  fInitAlarms(frmAlarm);
  
  // load Setup settings
  fLoadSetup(frmSetup);

  // load and initialize settings to hide/show forms
  fLoadHideShowSettings();

  // initialize all non-clock forms
  fInitAllForms();

  // startup clock displays
  fShowTime(frmClock);
  fLocaleTime(frmLocation, frmTimeZones);
  
}